"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class StatusAttribution(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Type:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[StatusAttribution._Type.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: StatusAttribution._Type.ValueType  # 0
        RESHARE: StatusAttribution._Type.ValueType  # 1
        EXTERNAL_SHARE: StatusAttribution._Type.ValueType  # 2
        MUSIC: StatusAttribution._Type.ValueType  # 3
        STATUS_MENTION: StatusAttribution._Type.ValueType  # 4
        GROUP_STATUS: StatusAttribution._Type.ValueType  # 5
        RL_ATTRIBUTION: StatusAttribution._Type.ValueType  # 6
        AI_CREATED: StatusAttribution._Type.ValueType  # 7

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    UNKNOWN: StatusAttribution.Type.ValueType  # 0
    RESHARE: StatusAttribution.Type.ValueType  # 1
    EXTERNAL_SHARE: StatusAttribution.Type.ValueType  # 2
    MUSIC: StatusAttribution.Type.ValueType  # 3
    STATUS_MENTION: StatusAttribution.Type.ValueType  # 4
    GROUP_STATUS: StatusAttribution.Type.ValueType  # 5
    RL_ATTRIBUTION: StatusAttribution.Type.ValueType  # 6
    AI_CREATED: StatusAttribution.Type.ValueType  # 7

    @typing.final
    class AiCreatedAttribution(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _Source:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _SourceEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[StatusAttribution.AiCreatedAttribution._Source.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            UNKNOWN: StatusAttribution.AiCreatedAttribution._Source.ValueType  # 0
            STATUS_MIMICRY: StatusAttribution.AiCreatedAttribution._Source.ValueType  # 1

        class Source(_Source, metaclass=_SourceEnumTypeWrapper): ...
        UNKNOWN: StatusAttribution.AiCreatedAttribution.Source.ValueType  # 0
        STATUS_MIMICRY: StatusAttribution.AiCreatedAttribution.Source.ValueType  # 1

        SOURCE_FIELD_NUMBER: builtins.int
        source: global___StatusAttribution.AiCreatedAttribution.Source.ValueType
        def __init__(
            self,
            *,
            source: global___StatusAttribution.AiCreatedAttribution.Source.ValueType | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["source", b"source"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["source", b"source"]) -> None: ...

    @typing.final
    class RLAttribution(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _Source:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _SourceEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[StatusAttribution.RLAttribution._Source.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            UNKNOWN: StatusAttribution.RLAttribution._Source.ValueType  # 0
            RAY_BAN_META_GLASSES: StatusAttribution.RLAttribution._Source.ValueType  # 1
            OAKLEY_META_GLASSES: StatusAttribution.RLAttribution._Source.ValueType  # 2
            HYPERNOVA_GLASSES: StatusAttribution.RLAttribution._Source.ValueType  # 3

        class Source(_Source, metaclass=_SourceEnumTypeWrapper): ...
        UNKNOWN: StatusAttribution.RLAttribution.Source.ValueType  # 0
        RAY_BAN_META_GLASSES: StatusAttribution.RLAttribution.Source.ValueType  # 1
        OAKLEY_META_GLASSES: StatusAttribution.RLAttribution.Source.ValueType  # 2
        HYPERNOVA_GLASSES: StatusAttribution.RLAttribution.Source.ValueType  # 3

        SOURCE_FIELD_NUMBER: builtins.int
        source: global___StatusAttribution.RLAttribution.Source.ValueType
        def __init__(
            self,
            *,
            source: global___StatusAttribution.RLAttribution.Source.ValueType | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["source", b"source"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["source", b"source"]) -> None: ...

    @typing.final
    class ExternalShare(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _Source:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _SourceEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[StatusAttribution.ExternalShare._Source.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            UNKNOWN: StatusAttribution.ExternalShare._Source.ValueType  # 0
            INSTAGRAM: StatusAttribution.ExternalShare._Source.ValueType  # 1
            FACEBOOK: StatusAttribution.ExternalShare._Source.ValueType  # 2
            MESSENGER: StatusAttribution.ExternalShare._Source.ValueType  # 3
            SPOTIFY: StatusAttribution.ExternalShare._Source.ValueType  # 4
            YOUTUBE: StatusAttribution.ExternalShare._Source.ValueType  # 5
            PINTEREST: StatusAttribution.ExternalShare._Source.ValueType  # 6
            THREADS: StatusAttribution.ExternalShare._Source.ValueType  # 7

        class Source(_Source, metaclass=_SourceEnumTypeWrapper): ...
        UNKNOWN: StatusAttribution.ExternalShare.Source.ValueType  # 0
        INSTAGRAM: StatusAttribution.ExternalShare.Source.ValueType  # 1
        FACEBOOK: StatusAttribution.ExternalShare.Source.ValueType  # 2
        MESSENGER: StatusAttribution.ExternalShare.Source.ValueType  # 3
        SPOTIFY: StatusAttribution.ExternalShare.Source.ValueType  # 4
        YOUTUBE: StatusAttribution.ExternalShare.Source.ValueType  # 5
        PINTEREST: StatusAttribution.ExternalShare.Source.ValueType  # 6
        THREADS: StatusAttribution.ExternalShare.Source.ValueType  # 7

        ACTIONURL_FIELD_NUMBER: builtins.int
        SOURCE_FIELD_NUMBER: builtins.int
        DURATION_FIELD_NUMBER: builtins.int
        ACTIONFALLBACKURL_FIELD_NUMBER: builtins.int
        actionURL: builtins.str
        source: global___StatusAttribution.ExternalShare.Source.ValueType
        duration: builtins.int
        actionFallbackURL: builtins.str
        def __init__(
            self,
            *,
            actionURL: builtins.str | None = ...,
            source: global___StatusAttribution.ExternalShare.Source.ValueType | None = ...,
            duration: builtins.int | None = ...,
            actionFallbackURL: builtins.str | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["actionFallbackURL", b"actionFallbackURL", "actionURL", b"actionURL", "duration", b"duration", "source", b"source"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["actionFallbackURL", b"actionFallbackURL", "actionURL", b"actionURL", "duration", b"duration", "source", b"source"]) -> None: ...

    @typing.final
    class StatusReshare(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _Source:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _SourceEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[StatusAttribution.StatusReshare._Source.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            UNKNOWN: StatusAttribution.StatusReshare._Source.ValueType  # 0
            INTERNAL_RESHARE: StatusAttribution.StatusReshare._Source.ValueType  # 1
            MENTION_RESHARE: StatusAttribution.StatusReshare._Source.ValueType  # 2
            CHANNEL_RESHARE: StatusAttribution.StatusReshare._Source.ValueType  # 3

        class Source(_Source, metaclass=_SourceEnumTypeWrapper): ...
        UNKNOWN: StatusAttribution.StatusReshare.Source.ValueType  # 0
        INTERNAL_RESHARE: StatusAttribution.StatusReshare.Source.ValueType  # 1
        MENTION_RESHARE: StatusAttribution.StatusReshare.Source.ValueType  # 2
        CHANNEL_RESHARE: StatusAttribution.StatusReshare.Source.ValueType  # 3

        @typing.final
        class Metadata(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            DURATION_FIELD_NUMBER: builtins.int
            CHANNELJID_FIELD_NUMBER: builtins.int
            CHANNELMESSAGEID_FIELD_NUMBER: builtins.int
            HASMULTIPLERESHARES_FIELD_NUMBER: builtins.int
            duration: builtins.int
            channelJID: builtins.str
            channelMessageID: builtins.int
            hasMultipleReshares: builtins.bool
            def __init__(
                self,
                *,
                duration: builtins.int | None = ...,
                channelJID: builtins.str | None = ...,
                channelMessageID: builtins.int | None = ...,
                hasMultipleReshares: builtins.bool | None = ...,
            ) -> None: ...
            def HasField(self, field_name: typing.Literal["channelJID", b"channelJID", "channelMessageID", b"channelMessageID", "duration", b"duration", "hasMultipleReshares", b"hasMultipleReshares"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing.Literal["channelJID", b"channelJID", "channelMessageID", b"channelMessageID", "duration", b"duration", "hasMultipleReshares", b"hasMultipleReshares"]) -> None: ...

        SOURCE_FIELD_NUMBER: builtins.int
        METADATA_FIELD_NUMBER: builtins.int
        source: global___StatusAttribution.StatusReshare.Source.ValueType
        @property
        def metadata(self) -> global___StatusAttribution.StatusReshare.Metadata: ...
        def __init__(
            self,
            *,
            source: global___StatusAttribution.StatusReshare.Source.ValueType | None = ...,
            metadata: global___StatusAttribution.StatusReshare.Metadata | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["metadata", b"metadata", "source", b"source"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["metadata", b"metadata", "source", b"source"]) -> None: ...

    @typing.final
    class GroupStatus(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        AUTHORJID_FIELD_NUMBER: builtins.int
        authorJID: builtins.str
        def __init__(
            self,
            *,
            authorJID: builtins.str | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["authorJID", b"authorJID"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["authorJID", b"authorJID"]) -> None: ...

    @typing.final
    class Music(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        AUTHORNAME_FIELD_NUMBER: builtins.int
        SONGID_FIELD_NUMBER: builtins.int
        TITLE_FIELD_NUMBER: builtins.int
        AUTHOR_FIELD_NUMBER: builtins.int
        ARTISTATTRIBUTION_FIELD_NUMBER: builtins.int
        ISEXPLICIT_FIELD_NUMBER: builtins.int
        authorName: builtins.str
        songID: builtins.str
        title: builtins.str
        author: builtins.str
        artistAttribution: builtins.str
        isExplicit: builtins.bool
        def __init__(
            self,
            *,
            authorName: builtins.str | None = ...,
            songID: builtins.str | None = ...,
            title: builtins.str | None = ...,
            author: builtins.str | None = ...,
            artistAttribution: builtins.str | None = ...,
            isExplicit: builtins.bool | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["artistAttribution", b"artistAttribution", "author", b"author", "authorName", b"authorName", "isExplicit", b"isExplicit", "songID", b"songID", "title", b"title"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["artistAttribution", b"artistAttribution", "author", b"author", "authorName", b"authorName", "isExplicit", b"isExplicit", "songID", b"songID", "title", b"title"]) -> None: ...

    STATUSRESHARE_FIELD_NUMBER: builtins.int
    EXTERNALSHARE_FIELD_NUMBER: builtins.int
    MUSIC_FIELD_NUMBER: builtins.int
    GROUPSTATUS_FIELD_NUMBER: builtins.int
    RLATTRIBUTION_FIELD_NUMBER: builtins.int
    AICREATEDATTRIBUTION_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    ACTIONURL_FIELD_NUMBER: builtins.int
    type: global___StatusAttribution.Type.ValueType
    actionURL: builtins.str
    @property
    def statusReshare(self) -> global___StatusAttribution.StatusReshare: ...
    @property
    def externalShare(self) -> global___StatusAttribution.ExternalShare: ...
    @property
    def music(self) -> global___StatusAttribution.Music: ...
    @property
    def groupStatus(self) -> global___StatusAttribution.GroupStatus: ...
    @property
    def rlAttribution(self) -> global___StatusAttribution.RLAttribution: ...
    @property
    def aiCreatedAttribution(self) -> global___StatusAttribution.AiCreatedAttribution: ...
    def __init__(
        self,
        *,
        statusReshare: global___StatusAttribution.StatusReshare | None = ...,
        externalShare: global___StatusAttribution.ExternalShare | None = ...,
        music: global___StatusAttribution.Music | None = ...,
        groupStatus: global___StatusAttribution.GroupStatus | None = ...,
        rlAttribution: global___StatusAttribution.RLAttribution | None = ...,
        aiCreatedAttribution: global___StatusAttribution.AiCreatedAttribution | None = ...,
        type: global___StatusAttribution.Type.ValueType | None = ...,
        actionURL: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["actionURL", b"actionURL", "aiCreatedAttribution", b"aiCreatedAttribution", "attributionData", b"attributionData", "externalShare", b"externalShare", "groupStatus", b"groupStatus", "music", b"music", "rlAttribution", b"rlAttribution", "statusReshare", b"statusReshare", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["actionURL", b"actionURL", "aiCreatedAttribution", b"aiCreatedAttribution", "attributionData", b"attributionData", "externalShare", b"externalShare", "groupStatus", b"groupStatus", "music", b"music", "rlAttribution", b"rlAttribution", "statusReshare", b"statusReshare", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["attributionData", b"attributionData"]) -> typing.Literal["statusReshare", "externalShare", "music", "groupStatus", "rlAttribution", "aiCreatedAttribution"] | None: ...

global___StatusAttribution = StatusAttribution



from __future__ import annotations

import ctypes
import logging
from threading import Event as EventThread
from typing import TYPE_CHECKING, Callable, Dict, Type, TypeVar

import segno
from google.protobuf.message import Message

from neonize.exc import UnsupportedEvent

from .proto.Neonize_pb2 import QR as QREv
from .proto.Neonize_pb2 import BlocklistChange as BlocklistChangeEv
from .proto.Neonize_pb2 import BlocklistEvent as BlocklistEv
from .proto.Neonize_pb2 import CallAccept as CallAcceptEv
from .proto.Neonize_pb2 import CallOffer as CallOfferEv
from .proto.Neonize_pb2 import CallOfferNotice as CallOfferNoticeEv
from .proto.Neonize_pb2 import CallPreAccept as CallPreAcceptEv
from .proto.Neonize_pb2 import CallRelayLatency as CallRelayLatencyEV
from .proto.Neonize_pb2 import CallTerminate as CallTerminateEv
from .proto.Neonize_pb2 import CallTransport as CallTransportEv
from .proto.Neonize_pb2 import ChatPresence as ChatPresenceEv
from .proto.Neonize_pb2 import ClientOutdated as ClientOutdatedEv
from .proto.Neonize_pb2 import Connected as ConnectedEv
from .proto.Neonize_pb2 import ConnectFailure as ConnectFailureEv
from .proto.Neonize_pb2 import Device
from .proto.Neonize_pb2 import Disconnected as DisconnectedEv
from .proto.Neonize_pb2 import GroupInfoEvent as GroupInfoEv
from .proto.Neonize_pb2 import HistorySync as HistorySyncEv
from .proto.Neonize_pb2 import IdentityChange as IdentityChangeEv
from .proto.Neonize_pb2 import JoinedGroup as JoinedGroupEv
from .proto.Neonize_pb2 import KeepAliveRestored as KeepAliveRestoredEv
from .proto.Neonize_pb2 import KeepAliveTimeout as KeepAliveTimeoutEv
from .proto.Neonize_pb2 import LoggedOut as LoggedOutEv
from .proto.Neonize_pb2 import Message as MessageEv
from .proto.Neonize_pb2 import NewsletterJoin as NewsletterJoinEv
from .proto.Neonize_pb2 import NewsletterLeave as NewsletterLeaveEv
from .proto.Neonize_pb2 import NewsletterLiveUpdate as NewsletterLiveUpdateEV
from .proto.Neonize_pb2 import NewsLetterMessageMeta as NewsLetterMessageMetaEv
from .proto.Neonize_pb2 import NewsletterMuteChange as NewsletterMuteChangeEv
from .proto.Neonize_pb2 import OfflineSyncCompleted as OfflineSyncCompletedEv
from .proto.Neonize_pb2 import OfflineSyncPreview as OfflineSyncPreviewEv
from .proto.Neonize_pb2 import PairStatus as PairStatusEv
from .proto.Neonize_pb2 import Picture as PictureEv
from .proto.Neonize_pb2 import Presence as PresenceEv
from .proto.Neonize_pb2 import Receipt as ReceiptEv
from .proto.Neonize_pb2 import StreamError as StreamErrorEv
from .proto.Neonize_pb2 import StreamReplaced as StreamReplacedEv
from .proto.Neonize_pb2 import TemporaryBan as TemporaryBanEv
from .proto.Neonize_pb2 import UndecryptableMessage as UndecryptableMessageEv
from .proto.Neonize_pb2 import UnknownCallEvent as UnknownCallEventEV
from .proto.Neonize_pb2 import privacySettingsEvent as PrivacySettingsEv

log = logging.getLogger(__name__)
if TYPE_CHECKING:
    from .client import ClientFactory, NewClient
EventType = TypeVar("EventType", bound=Message)
EVENT_TO_INT: Dict[Type[Message], int] = {
    Device: 0,
    QREv: 1,
    PairStatusEv: 2,
    ConnectedEv: 3,
    KeepAliveTimeoutEv: 4,
    KeepAliveRestoredEv: 5,
    LoggedOutEv: 6,
    StreamReplacedEv: 7,
    TemporaryBanEv: 8,
    ConnectFailureEv: 9,
    ClientOutdatedEv: 10,
    StreamErrorEv: 11,
    DisconnectedEv: 12,
    HistorySyncEv: 13,
    NewsLetterMessageMetaEv: 16,
    MessageEv: 17,
    ReceiptEv: 18,
    ChatPresenceEv: 19,
    PresenceEv: 20,
    JoinedGroupEv: 21,
    GroupInfoEv: 22,
    PictureEv: 23,
    IdentityChangeEv: 24,
    PrivacySettingsEv: 25,
    OfflineSyncPreviewEv: 26,
    OfflineSyncCompletedEv: 27,
    BlocklistEv: 30,
    BlocklistChangeEv: 31,
    NewsletterJoinEv: 32,
    NewsletterLeaveEv: 33,
    NewsletterMuteChangeEv: 34,
    NewsletterLiveUpdateEV: 35,
    CallOfferEv: 36,
    CallAcceptEv: 37,
    CallPreAcceptEv: 38,
    CallTransportEv: 39,
    CallOfferNoticeEv: 40,
    CallRelayLatencyEV: 41,
    CallTerminateEv: 42,
    UnknownCallEventEV: 43,
    UndecryptableMessageEv: 44,
}
INT_TO_EVENT: Dict[int, Type[Message]] = {code: ev for ev, code in EVENT_TO_INT.items()}

event = EventThread()


class EventsManager:
    def __init__(self, client_factory: ClientFactory):
        self.client_factory = client_factory
        self.list_func: Dict[int, Callable[[NewClient, Message], None]] = {}

    def __call__(
        self, event: Type[EventType]
    ) -> Callable[[Callable[[NewClient, EventType], None]], None]:
        """
        Registers a callback function for a specific event type.

        :param event: The type of event to register the callback for.
        :type event: Type[EventType]
        :return: A decorator that registers the callback function.
        :rtype: Callablae[[Callable[[NewClient, EventType], None]], None]
        """

        def callback(func: Callable[[NewClient, EventType], None]) -> None:
            self.list_func.update({EVENT_TO_INT[event]: func})

        return callback


class Event:
    def __init__(self, client: NewClient):
        """
        Initializes the Event class with a client of type NewClient.
        Also sets up a default blocking function and an empty dictionary for list functions.

        :param client: An instance of the NewClient class
        :type client: NewClient
        """
        self.client = client
        self.paircode_cb = self.paircode(self.default_paircode_cb)
        self.list_func: Dict[int, Callable[[NewClient, Message], None]] = {}
        self._qr = self.__onqr

    def execute(
        self, uuid: int, binary: int, size: int, code: int
    ):  # Demands Attention
        """Executes a function from the list of functions based on the given code.

        :param binary: The binary data to be processed by the function.
        :type binary: int
        :param size: The size of the binary data.
        :type size: int
        :param code: The index of the function to be executed from the list of functions.
        :type code: int
        """
        if code not in INT_TO_EVENT:
            raise UnsupportedEvent()
        message = INT_TO_EVENT[code].FromString(ctypes.string_at(binary, size))
        if code == 0:
            self.client.me = message
            return
        elif code == 3:
            self.client.connected = True
        self.list_func[code](self.client, message)

    def __onqr(self, _: NewClient, data_qr: bytes):
        """
        Handles QR code generation and display.

        :param _: The client instance (not used in the function).
        :type _: NewClient
        :param data_qr: The data to be encoded in the QR code.
        :type data_qr: bytes
        """
        segno.make_qr(data_qr).terminal(compact=True)

    def qr(self, f: Callable[[NewClient, bytes], None]):
        """
        Sets a callback function for handling QR code data.

        :param f: The callback function that takes a NewClient instance and QR code data in bytes.
        :type f: Callable[[NewClient, bytes], None]
        """
        self._qr = f

    @property
    def paircode(self):
        def paircodecb(f: Callable[[NewClient, str, bool], None]):
            """
            Sets a callback function for handling pair codes.
            :param f: The callback function that takes a NewClient instance and pair code as a string.
            :type f: Callable[[NewClient, str], None]
            """

            def wrap_paircode_cb(code, connected):
                """
                Wraps the pair code callback function to include the client instance.
                :param code: The pair code as a string.
                :type code: str
                :param connected: A boolean indicating if the client is connected.
                :type connected: bool
                """
                paircode = ctypes.string_at(code)
                if self.paircode_cb:
                    f(self.client, paircode.decode(), connected)

            self.paircode_cb = wrap_paircode_cb
            return self.paircode_cb

        return paircodecb

    @staticmethod
    def default_paircode_cb(client: NewClient, data: str, connected: bool):
        if connected:
            log.info("authtenticated with pair code: %s", data)
        else:
            log.info("Pair code: %s", data)

    def __call__(
        self, event: Type[EventType]
    ) -> Callable[[Callable[[NewClient, EventType], None]], None]:
        """
        Registers a callback function for a specific event type.

        :param event: The type of event to register the callback for.
        :type event: Type[EventType]
        :return: A decorator that registers the callback function.
        :rtype: Callable[[Callable[[NewClient, EventType], None]], None]
        """

        def callback(func: Callable[[NewClient, EventType], None]) -> None:
            self.list_func.update({EVENT_TO_INT[event]: func})

        return callback



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: instamadilloCoreTypeActionLog/InstamadilloCoreTypeActionLog.proto
# Protobuf Python Version: 6.32.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    32,
    1,
    '',
    'instamadilloCoreTypeActionLog/InstamadilloCoreTypeActionLog.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAinstamadilloCoreTypeActionLog/InstamadilloCoreTypeActionLog.proto\x12\x1dInstamadilloCoreTypeActionLog\"n\n\tActionLog\x12M\n\x11\x61\x63tionLogReaction\x18\x01 \x01(\x0b\x32\x30.InstamadilloCoreTypeActionLog.ActionLogReactionH\x00\x42\x12\n\x10\x61\x63tionLogSubtype\")\n\x11\x41\x63tionLogReaction\x12\x14\n\x0c\x65mojiUnicode\x18\x01 \x01(\tB9Z7go.mau.fi/whatsmeow/proto/instamadilloCoreTypeActionLog')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'instamadilloCoreTypeActionLog.InstamadilloCoreTypeActionLog_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z7go.mau.fi/whatsmeow/proto/instamadilloCoreTypeActionLog'
  _globals['_ACTIONLOG']._serialized_start=100
  _globals['_ACTIONLOG']._serialized_end=210
  _globals['_ACTIONLOGREACTION']._serialized_start=212
  _globals['_ACTIONLOGREACTION']._serialized_end=253
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _PjpegScanConfiguration:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _PjpegScanConfigurationEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PjpegScanConfiguration.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    PJPEG_SCAN_CONFIGURATION_UNSPECIFIED: _PjpegScanConfiguration.ValueType  # 0
    PJPEG_SCAN_CONFIGURATION_WA: _PjpegScanConfiguration.ValueType  # 1
    PJPEG_SCAN_CONFIGURATION_E15: _PjpegScanConfiguration.ValueType  # 2
    PJPEG_SCAN_CONFIGURATION_E35: _PjpegScanConfiguration.ValueType  # 3

class PjpegScanConfiguration(_PjpegScanConfiguration, metaclass=_PjpegScanConfigurationEnumTypeWrapper): ...

PJPEG_SCAN_CONFIGURATION_UNSPECIFIED: PjpegScanConfiguration.ValueType  # 0
PJPEG_SCAN_CONFIGURATION_WA: PjpegScanConfiguration.ValueType  # 1
PJPEG_SCAN_CONFIGURATION_E15: PjpegScanConfiguration.ValueType  # 2
PJPEG_SCAN_CONFIGURATION_E35: PjpegScanConfiguration.ValueType  # 3
global___PjpegScanConfiguration = PjpegScanConfiguration

@typing.final
class Media(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _InterventionType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _InterventionTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Media._InterventionType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSET: Media._InterventionType.ValueType  # 0
        NONE: Media._InterventionType.ValueType  # 1
        NUDE: Media._InterventionType.ValueType  # 2

    class InterventionType(_InterventionType, metaclass=_InterventionTypeEnumTypeWrapper): ...
    UNSET: Media.InterventionType.ValueType  # 0
    NONE: Media.InterventionType.ValueType  # 1
    NUDE: Media.InterventionType.ValueType  # 2

    STATICPHOTO_FIELD_NUMBER: builtins.int
    VOICE_FIELD_NUMBER: builtins.int
    VIDEO_FIELD_NUMBER: builtins.int
    RAVEN_FIELD_NUMBER: builtins.int
    GIF_FIELD_NUMBER: builtins.int
    AVATARSTICKER_FIELD_NUMBER: builtins.int
    @property
    def staticPhoto(self) -> global___StaticPhoto: ...
    @property
    def voice(self) -> global___Voice: ...
    @property
    def video(self) -> global___Video: ...
    @property
    def raven(self) -> global___Raven: ...
    @property
    def gif(self) -> global___Gif: ...
    @property
    def avatarSticker(self) -> global___AvatarSticker: ...
    def __init__(
        self,
        *,
        staticPhoto: global___StaticPhoto | None = ...,
        voice: global___Voice | None = ...,
        video: global___Video | None = ...,
        raven: global___Raven | None = ...,
        gif: global___Gif | None = ...,
        avatarSticker: global___AvatarSticker | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["avatarSticker", b"avatarSticker", "gif", b"gif", "media", b"media", "raven", b"raven", "staticPhoto", b"staticPhoto", "video", b"video", "voice", b"voice"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["avatarSticker", b"avatarSticker", "gif", b"gif", "media", b"media", "raven", b"raven", "staticPhoto", b"staticPhoto", "video", b"video", "voice", b"voice"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["media", b"media"]) -> typing.Literal["staticPhoto", "voice", "video", "raven", "gif", "avatarSticker"] | None: ...

global___Media = Media

@typing.final
class StaticPhoto(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIATRANSPORT_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    WIDTH_FIELD_NUMBER: builtins.int
    SCANLENGTHS_FIELD_NUMBER: builtins.int
    THUMBNAIL_FIELD_NUMBER: builtins.int
    PJPEGSCANCONFIGURATION_FIELD_NUMBER: builtins.int
    height: builtins.int
    width: builtins.int
    pjpegScanConfiguration: global___PjpegScanConfiguration.ValueType
    @property
    def mediaTransport(self) -> global___CommonMediaTransport: ...
    @property
    def scanLengths(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def thumbnail(self) -> global___Thumbnail: ...
    def __init__(
        self,
        *,
        mediaTransport: global___CommonMediaTransport | None = ...,
        height: builtins.int | None = ...,
        width: builtins.int | None = ...,
        scanLengths: collections.abc.Iterable[builtins.int] | None = ...,
        thumbnail: global___Thumbnail | None = ...,
        pjpegScanConfiguration: global___PjpegScanConfiguration.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["height", b"height", "mediaTransport", b"mediaTransport", "pjpegScanConfiguration", b"pjpegScanConfiguration", "thumbnail", b"thumbnail", "width", b"width"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["height", b"height", "mediaTransport", b"mediaTransport", "pjpegScanConfiguration", b"pjpegScanConfiguration", "scanLengths", b"scanLengths", "thumbnail", b"thumbnail", "width", b"width"]) -> None: ...

global___StaticPhoto = StaticPhoto

@typing.final
class Voice(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIATRANSPORT_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    WAVEFORMS_FIELD_NUMBER: builtins.int
    WAVEFORMSAMPLINGFREQUENCYHZ_FIELD_NUMBER: builtins.int
    duration: builtins.int
    waveformSamplingFrequencyHz: builtins.int
    @property
    def mediaTransport(self) -> global___CommonMediaTransport: ...
    @property
    def waveforms(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        mediaTransport: global___CommonMediaTransport | None = ...,
        duration: builtins.int | None = ...,
        waveforms: collections.abc.Iterable[builtins.float] | None = ...,
        waveformSamplingFrequencyHz: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["duration", b"duration", "mediaTransport", b"mediaTransport", "waveformSamplingFrequencyHz", b"waveformSamplingFrequencyHz"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["duration", b"duration", "mediaTransport", b"mediaTransport", "waveformSamplingFrequencyHz", b"waveformSamplingFrequencyHz", "waveforms", b"waveforms"]) -> None: ...

global___Voice = Voice

@typing.final
class Video(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIATRANSPORT_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    WIDTH_FIELD_NUMBER: builtins.int
    THUMBNAIL_FIELD_NUMBER: builtins.int
    VIDEOEXTRAMETADATA_FIELD_NUMBER: builtins.int
    height: builtins.int
    width: builtins.int
    @property
    def mediaTransport(self) -> global___CommonMediaTransport: ...
    @property
    def thumbnail(self) -> global___Thumbnail: ...
    @property
    def videoExtraMetadata(self) -> global___VideoExtraMetadata: ...
    def __init__(
        self,
        *,
        mediaTransport: global___CommonMediaTransport | None = ...,
        height: builtins.int | None = ...,
        width: builtins.int | None = ...,
        thumbnail: global___Thumbnail | None = ...,
        videoExtraMetadata: global___VideoExtraMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["height", b"height", "mediaTransport", b"mediaTransport", "thumbnail", b"thumbnail", "videoExtraMetadata", b"videoExtraMetadata", "width", b"width"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["height", b"height", "mediaTransport", b"mediaTransport", "thumbnail", b"thumbnail", "videoExtraMetadata", b"videoExtraMetadata", "width", b"width"]) -> None: ...

global___Video = Video

@typing.final
class Gif(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIATRANSPORT_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    WIDTH_FIELD_NUMBER: builtins.int
    ISSTICKER_FIELD_NUMBER: builtins.int
    STICKERID_FIELD_NUMBER: builtins.int
    GIFURL_FIELD_NUMBER: builtins.int
    GIFSIZE_FIELD_NUMBER: builtins.int
    ISRANDOM_FIELD_NUMBER: builtins.int
    height: builtins.int
    width: builtins.int
    isSticker: builtins.bool
    stickerID: builtins.str
    gifURL: builtins.str
    gifSize: builtins.int
    isRandom: builtins.bool
    @property
    def mediaTransport(self) -> global___CommonMediaTransport: ...
    def __init__(
        self,
        *,
        mediaTransport: global___CommonMediaTransport | None = ...,
        height: builtins.int | None = ...,
        width: builtins.int | None = ...,
        isSticker: builtins.bool | None = ...,
        stickerID: builtins.str | None = ...,
        gifURL: builtins.str | None = ...,
        gifSize: builtins.int | None = ...,
        isRandom: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["gifSize", b"gifSize", "gifURL", b"gifURL", "height", b"height", "isRandom", b"isRandom", "isSticker", b"isSticker", "mediaTransport", b"mediaTransport", "stickerID", b"stickerID", "width", b"width"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["gifSize", b"gifSize", "gifURL", b"gifURL", "height", b"height", "isRandom", b"isRandom", "isSticker", b"isSticker", "mediaTransport", b"mediaTransport", "stickerID", b"stickerID", "width", b"width"]) -> None: ...

global___Gif = Gif

@typing.final
class AvatarSticker(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIATRANSPORT_FIELD_NUMBER: builtins.int
    ISANIMATED_FIELD_NUMBER: builtins.int
    STICKERID_FIELD_NUMBER: builtins.int
    STICKERTEMPLATE_FIELD_NUMBER: builtins.int
    NUXTYPE_FIELD_NUMBER: builtins.int
    isAnimated: builtins.bool
    stickerID: builtins.str
    stickerTemplate: builtins.str
    nuxType: builtins.int
    @property
    def mediaTransport(self) -> global___CommonMediaTransport: ...
    def __init__(
        self,
        *,
        mediaTransport: global___CommonMediaTransport | None = ...,
        isAnimated: builtins.bool | None = ...,
        stickerID: builtins.str | None = ...,
        stickerTemplate: builtins.str | None = ...,
        nuxType: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["isAnimated", b"isAnimated", "mediaTransport", b"mediaTransport", "nuxType", b"nuxType", "stickerID", b"stickerID", "stickerTemplate", b"stickerTemplate"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["isAnimated", b"isAnimated", "mediaTransport", b"mediaTransport", "nuxType", b"nuxType", "stickerID", b"stickerID", "stickerTemplate", b"stickerTemplate"]) -> None: ...

global___AvatarSticker = AvatarSticker

@typing.final
class Raven(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ViewMode:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ViewModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Raven._ViewMode.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        RAVEN_VIEW_MODEL_UNSPECIFIED: Raven._ViewMode.ValueType  # 0
        RAVEN_VIEW_MODEL_ONCE: Raven._ViewMode.ValueType  # 1
        RAVEN_VIEW_MODEL_REPLAYABLE: Raven._ViewMode.ValueType  # 2
        RAVEN_VIEW_MODEL_PERMANENT: Raven._ViewMode.ValueType  # 3

    class ViewMode(_ViewMode, metaclass=_ViewModeEnumTypeWrapper): ...
    RAVEN_VIEW_MODEL_UNSPECIFIED: Raven.ViewMode.ValueType  # 0
    RAVEN_VIEW_MODEL_ONCE: Raven.ViewMode.ValueType  # 1
    RAVEN_VIEW_MODEL_REPLAYABLE: Raven.ViewMode.ValueType  # 2
    RAVEN_VIEW_MODEL_PERMANENT: Raven.ViewMode.ValueType  # 3

    VIEWMODE_FIELD_NUMBER: builtins.int
    CONTENT_FIELD_NUMBER: builtins.int
    viewMode: global___Raven.ViewMode.ValueType
    @property
    def content(self) -> global___RavenContent: ...
    def __init__(
        self,
        *,
        viewMode: global___Raven.ViewMode.ValueType | None = ...,
        content: global___RavenContent | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["content", b"content", "viewMode", b"viewMode"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["content", b"content", "viewMode", b"viewMode"]) -> None: ...

global___Raven = Raven

@typing.final
class RavenContent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATICPHOTO_FIELD_NUMBER: builtins.int
    VIDEO_FIELD_NUMBER: builtins.int
    @property
    def staticPhoto(self) -> global___StaticPhoto: ...
    @property
    def video(self) -> global___Video: ...
    def __init__(
        self,
        *,
        staticPhoto: global___StaticPhoto | None = ...,
        video: global___Video | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["ravenContent", b"ravenContent", "staticPhoto", b"staticPhoto", "video", b"video"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["ravenContent", b"ravenContent", "staticPhoto", b"staticPhoto", "video", b"video"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["ravenContent", b"ravenContent"]) -> typing.Literal["staticPhoto", "video"] | None: ...

global___RavenContent = RavenContent

@typing.final
class Thumbnail(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIATRANSPORT_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    WIDTH_FIELD_NUMBER: builtins.int
    height: builtins.int
    width: builtins.int
    @property
    def mediaTransport(self) -> global___CommonMediaTransport: ...
    def __init__(
        self,
        *,
        mediaTransport: global___CommonMediaTransport | None = ...,
        height: builtins.int | None = ...,
        width: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["height", b"height", "mediaTransport", b"mediaTransport", "width", b"width"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["height", b"height", "mediaTransport", b"mediaTransport", "width", b"width"]) -> None: ...

global___Thumbnail = Thumbnail

@typing.final
class CommonMediaTransport(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIAID_FIELD_NUMBER: builtins.int
    FILESHA256_FIELD_NUMBER: builtins.int
    MEDIAKEY_FIELD_NUMBER: builtins.int
    FILEENCSHA256_FIELD_NUMBER: builtins.int
    DIRECTPATH_FIELD_NUMBER: builtins.int
    MEDIAKEYTIMESTAMP_FIELD_NUMBER: builtins.int
    SIDECAR_FIELD_NUMBER: builtins.int
    FILELENGTH_FIELD_NUMBER: builtins.int
    MIMETYPE_FIELD_NUMBER: builtins.int
    OBJECTID_FIELD_NUMBER: builtins.int
    mediaID: builtins.str
    fileSHA256: builtins.str
    mediaKey: builtins.str
    fileEncSHA256: builtins.str
    directPath: builtins.str
    mediaKeyTimestamp: builtins.str
    sidecar: builtins.str
    fileLength: builtins.int
    mimetype: builtins.str
    objectID: builtins.str
    def __init__(
        self,
        *,
        mediaID: builtins.str | None = ...,
        fileSHA256: builtins.str | None = ...,
        mediaKey: builtins.str | None = ...,
        fileEncSHA256: builtins.str | None = ...,
        directPath: builtins.str | None = ...,
        mediaKeyTimestamp: builtins.str | None = ...,
        sidecar: builtins.str | None = ...,
        fileLength: builtins.int | None = ...,
        mimetype: builtins.str | None = ...,
        objectID: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["directPath", b"directPath", "fileEncSHA256", b"fileEncSHA256", "fileLength", b"fileLength", "fileSHA256", b"fileSHA256", "mediaID", b"mediaID", "mediaKey", b"mediaKey", "mediaKeyTimestamp", b"mediaKeyTimestamp", "mimetype", b"mimetype", "objectID", b"objectID", "sidecar", b"sidecar"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["directPath", b"directPath", "fileEncSHA256", b"fileEncSHA256", "fileLength", b"fileLength", "fileSHA256", b"fileSHA256", "mediaID", b"mediaID", "mediaKey", b"mediaKey", "mediaKeyTimestamp", b"mediaKeyTimestamp", "mimetype", b"mimetype", "objectID", b"objectID", "sidecar", b"sidecar"]) -> None: ...

global___CommonMediaTransport = CommonMediaTransport

@typing.final
class VideoExtraMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UPLOADMOSCLIENTSCORE_FIELD_NUMBER: builtins.int
    uploadMosClientScore: builtins.float
    def __init__(
        self,
        *,
        uploadMosClientScore: builtins.float | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["uploadMosClientScore", b"uploadMosClientScore"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["uploadMosClientScore", b"uploadMosClientScore"]) -> None: ...

global___VideoExtraMetadata = VideoExtraMetadata



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ActionLog(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTIONLOGREACTION_FIELD_NUMBER: builtins.int
    @property
    def actionLogReaction(self) -> global___ActionLogReaction: ...
    def __init__(
        self,
        *,
        actionLogReaction: global___ActionLogReaction | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["actionLogReaction", b"actionLogReaction", "actionLogSubtype", b"actionLogSubtype"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["actionLogReaction", b"actionLogReaction", "actionLogSubtype", b"actionLogSubtype"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["actionLogSubtype", b"actionLogSubtype"]) -> typing.Literal["actionLogReaction"] | None: ...

global___ActionLog = ActionLog

@typing.final
class ActionLogReaction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EMOJIUNICODE_FIELD_NUMBER: builtins.int
    emojiUnicode: builtins.str
    def __init__(
        self,
        *,
        emojiUnicode: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["emojiUnicode", b"emojiUnicode"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["emojiUnicode", b"emojiUnicode"]) -> None: ...

global___ActionLogReaction = ActionLogReaction



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: instamadilloAddMessage/InstamadilloAddMessage.proto
# Protobuf Python Version: 6.32.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    32,
    1,
    '',
    'instamadilloAddMessage/InstamadilloAddMessage.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from instamadilloCoreTypeActionLog import InstamadilloCoreTypeActionLog_pb2 as instamadilloCoreTypeActionLog_dot_InstamadilloCoreTypeActionLog__pb2
from instamadilloCoreTypeAdminMessage import InstamadilloCoreTypeAdminMessage_pb2 as instamadilloCoreTypeAdminMessage_dot_InstamadilloCoreTypeAdminMessage__pb2
from instamadilloCoreTypeCollection import InstamadilloCoreTypeCollection_pb2 as instamadilloCoreTypeCollection_dot_InstamadilloCoreTypeCollection__pb2
from instamadilloCoreTypeLink import InstamadilloCoreTypeLink_pb2 as instamadilloCoreTypeLink_dot_InstamadilloCoreTypeLink__pb2
from instamadilloCoreTypeMedia import InstamadilloCoreTypeMedia_pb2 as instamadilloCoreTypeMedia_dot_InstamadilloCoreTypeMedia__pb2
from instamadilloCoreTypeText import InstamadilloCoreTypeText_pb2 as instamadilloCoreTypeText_dot_InstamadilloCoreTypeText__pb2
from instamadilloXmaContentRef import InstamadilloXmaContentRef_pb2 as instamadilloXmaContentRef_dot_InstamadilloXmaContentRef__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3instamadilloAddMessage/InstamadilloAddMessage.proto\x12\x16InstamadilloAddMessage\x1a\x41instamadilloCoreTypeActionLog/InstamadilloCoreTypeActionLog.proto\x1aGinstamadilloCoreTypeAdminMessage/InstamadilloCoreTypeAdminMessage.proto\x1a\x43instamadilloCoreTypeCollection/InstamadilloCoreTypeCollection.proto\x1a\x37instamadilloCoreTypeLink/InstamadilloCoreTypeLink.proto\x1a\x39instamadilloCoreTypeMedia/InstamadilloCoreTypeMedia.proto\x1a\x37instamadilloCoreTypeText/InstamadilloCoreTypeText.proto\x1a\x39instamadilloXmaContentRef/InstamadilloXmaContentRef.proto\"\x8d\x01\n\x11\x41\x64\x64MessagePayload\x12:\n\x07\x63ontent\x18\x01 \x01(\x0b\x32).InstamadilloAddMessage.AddMessageContent\x12<\n\x08metadata\x18\x02 \x01(\x0b\x32*.InstamadilloAddMessage.AddMessageMetadata\"\xb4\x04\n\x11\x41\x64\x64MessageContent\x12.\n\x04text\x18\x01 \x01(\x0b\x32\x1e.InstamadilloCoreTypeText.TextH\x00\x12,\n\x04like\x18\x02 \x01(\x0b\x32\x1c.InstamadilloAddMessage.LikeH\x00\x12.\n\x04link\x18\x03 \x01(\x0b\x32\x1e.InstamadilloCoreTypeLink.LinkH\x00\x12\x44\n\x10receiverFetchXma\x18\x04 \x01(\x0b\x32(.InstamadilloAddMessage.ReceiverFetchXmaH\x00\x12\x31\n\x05media\x18\x05 \x01(\x0b\x32 .InstamadilloCoreTypeMedia.MediaH\x00\x12:\n\x0bplaceholder\x18\x06 \x01(\x0b\x32#.InstamadilloAddMessage.PlaceholderH\x00\x12@\n\ncollection\x18\x07 \x01(\x0b\x32*.InstamadilloCoreTypeCollection.CollectionH\x00\x12\x46\n\x0c\x61\x64minMessage\x18\x08 \x01(\x0b\x32..InstamadilloCoreTypeAdminMessage.AdminMessageH\x00\x12=\n\tactionLog\x18\t \x01(\x0b\x32(.InstamadilloCoreTypeActionLog.ActionLogH\x00\x42\x13\n\x11\x61\x64\x64MessageContent\"\xbe\x02\n\x12\x41\x64\x64MessageMetadata\x12\x14\n\x0csendSilently\x18\x01 \x01(\x08\x12\x42\n\x10privateReplyInfo\x18\x02 \x01(\x0b\x32(.InstamadilloAddMessage.PrivateReplyInfo\x12\x42\n\x10repliedToMessage\x18\x03 \x01(\x0b\x32(.InstamadilloAddMessage.RepliedToMessage\x12\x42\n\x10\x66orwardingParams\x18\x04 \x01(\x0b\x32(.InstamadilloAddMessage.ForwardingParams\x12\x46\n\x12\x65phemeralityParams\x18\x05 \x01(\x0b\x32*.InstamadilloAddMessage.EphemeralityParams\"\xd2\x01\n\x10RepliedToMessage\x12\x1c\n\x14repliedToMessageOtid\x18\x01 \x01(\t\x12\'\n\x1frepliedToMessageWaServerTimeSec\x18\x02 \x01(\t\x12(\n repliedToMessageCollectionItemID\x18\x03 \x01(\t\x12M\n\x0comMicroSecTS\x18\x04 \x01(\x0b\x32\x37.InstamadilloAddMessage.OpenMessageMicroSecondTimestamp\"P\n\x1fOpenMessageMicroSecondTimestamp\x12\x13\n\x0btimestampMS\x18\x01 \x01(\x03\x12\x18\n\x10microSecondsBits\x18\x02 \x01(\x05\"7\n\x10PrivateReplyInfo\x12\x11\n\tcommentID\x18\x01 \x01(\t\x12\x10\n\x08postLink\x18\x02 \x01(\t\"-\n\x10\x46orwardingParams\x12\x19\n\x11\x66orwardedThreadID\x18\x01 \x01(\t\"2\n\x12\x45phemeralityParams\x12\x1c\n\x14\x65phemeralDurationSec\x18\x01 \x01(\x03\"\x06\n\x04Like\"\xa6\x01\n\x10ReceiverFetchXma\x12\x12\n\ncontentRef\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\x12/\n\x05media\x18\x03 \x01(\x0b\x32 .InstamadilloCoreTypeMedia.Media\x12?\n\rxmaContentRef\x18\x04 \x01(\x0b\x32(.InstamadilloXmaContentRef.XmaContentRef\"\xaa\x02\n\x0bPlaceholder\x12\x41\n\x0fplaceholderType\x18\x01 \x01(\x0e\x32(.InstamadilloAddMessage.Placeholder.Type\"\xd7\x01\n\x04Type\x12\x19\n\x15PLACEHOLDER_TYPE_NONE\x10\x00\x12\'\n#PLACEHOLDER_TYPE_DECRYPTION_FAILURE\x10\x01\x12.\n*PLACEHOLDER_TYPE_NOT_SUPPORTED_NEED_UPDATE\x10\x02\x12\'\n#PLACEHOLDER_TYPE_DEVICE_UNAVAILABLE\x10\x03\x12\x32\n.PLACEHOLDER_TYPE_NOT_SUPPORTED_NOT_RECOVERABLE\x10\x04\x42\x32Z0go.mau.fi/whatsmeow/proto/instamadilloAddMessage')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'instamadilloAddMessage.InstamadilloAddMessage_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z0go.mau.fi/whatsmeow/proto/instamadilloAddMessage'
  _globals['_ADDMESSAGEPAYLOAD']._serialized_start=521
  _globals['_ADDMESSAGEPAYLOAD']._serialized_end=662
  _globals['_ADDMESSAGECONTENT']._serialized_start=665
  _globals['_ADDMESSAGECONTENT']._serialized_end=1229
  _globals['_ADDMESSAGEMETADATA']._serialized_start=1232
  _globals['_ADDMESSAGEMETADATA']._serialized_end=1550
  _globals['_REPLIEDTOMESSAGE']._serialized_start=1553
  _globals['_REPLIEDTOMESSAGE']._serialized_end=1763
  _globals['_OPENMESSAGEMICROSECONDTIMESTAMP']._serialized_start=1765
  _globals['_OPENMESSAGEMICROSECONDTIMESTAMP']._serialized_end=1845
  _globals['_PRIVATEREPLYINFO']._serialized_start=1847
  _globals['_PRIVATEREPLYINFO']._serialized_end=1902
  _globals['_FORWARDINGPARAMS']._serialized_start=1904
  _globals['_FORWARDINGPARAMS']._serialized_end=1949
  _globals['_EPHEMERALITYPARAMS']._serialized_start=1951
  _globals['_EPHEMERALITYPARAMS']._serialized_end=2001
  _globals['_LIKE']._serialized_start=2003
  _globals['_LIKE']._serialized_end=2009
  _globals['_RECEIVERFETCHXMA']._serialized_start=2012
  _globals['_RECEIVERFETCHXMA']._serialized_end=2178
  _globals['_PLACEHOLDER']._serialized_start=2181
  _globals['_PLACEHOLDER']._serialized_end=2479
  _globals['_PLACEHOLDER_TYPE']._serialized_start=2264
  _globals['_PLACEHOLDER_TYPE']._serialized_end=2479
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import instamadilloCoreTypeMedia.InstamadilloCoreTypeMedia_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SupplementMessagePayload(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TARGETMESSAGEOTID_FIELD_NUMBER: builtins.int
    UNIQUINGKEYFORSUPPLEMENTALDATA_FIELD_NUMBER: builtins.int
    CONTENT_FIELD_NUMBER: builtins.int
    TARGETMESSAGEWASERVERTIMESEC_FIELD_NUMBER: builtins.int
    TARGETWATHREADID_FIELD_NUMBER: builtins.int
    targetMessageOtid: builtins.str
    uniquingKeyForSupplementalData: builtins.str
    targetMessageWaServerTimeSec: builtins.str
    targetWaThreadID: builtins.str
    @property
    def content(self) -> global___SupplementMessageContent: ...
    def __init__(
        self,
        *,
        targetMessageOtid: builtins.str | None = ...,
        uniquingKeyForSupplementalData: builtins.str | None = ...,
        content: global___SupplementMessageContent | None = ...,
        targetMessageWaServerTimeSec: builtins.str | None = ...,
        targetWaThreadID: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["content", b"content", "targetMessageOtid", b"targetMessageOtid", "targetMessageWaServerTimeSec", b"targetMessageWaServerTimeSec", "targetWaThreadID", b"targetWaThreadID", "uniquingKeyForSupplementalData", b"uniquingKeyForSupplementalData"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["content", b"content", "targetMessageOtid", b"targetMessageOtid", "targetMessageWaServerTimeSec", b"targetMessageWaServerTimeSec", "targetWaThreadID", b"targetWaThreadID", "uniquingKeyForSupplementalData", b"uniquingKeyForSupplementalData"]) -> None: ...

global___SupplementMessagePayload = SupplementMessagePayload

@typing.final
class SupplementMessageContent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REACTION_FIELD_NUMBER: builtins.int
    CONTENTVIEW_FIELD_NUMBER: builtins.int
    EDITTEXT_FIELD_NUMBER: builtins.int
    MEDIAREACTION_FIELD_NUMBER: builtins.int
    ORIGINALTRANSPORTPAYLOAD_FIELD_NUMBER: builtins.int
    MEDIAINTERVENTIONS_FIELD_NUMBER: builtins.int
    @property
    def reaction(self) -> global___Reaction: ...
    @property
    def contentView(self) -> global___ContentView: ...
    @property
    def editText(self) -> global___EditText: ...
    @property
    def mediaReaction(self) -> global___MediaReaction: ...
    @property
    def originalTransportPayload(self) -> global___OriginalTransportPayload: ...
    @property
    def mediaInterventions(self) -> global___MediaInterventions: ...
    def __init__(
        self,
        *,
        reaction: global___Reaction | None = ...,
        contentView: global___ContentView | None = ...,
        editText: global___EditText | None = ...,
        mediaReaction: global___MediaReaction | None = ...,
        originalTransportPayload: global___OriginalTransportPayload | None = ...,
        mediaInterventions: global___MediaInterventions | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["contentView", b"contentView", "editText", b"editText", "mediaInterventions", b"mediaInterventions", "mediaReaction", b"mediaReaction", "originalTransportPayload", b"originalTransportPayload", "reaction", b"reaction", "supplementMessageContent", b"supplementMessageContent"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["contentView", b"contentView", "editText", b"editText", "mediaInterventions", b"mediaInterventions", "mediaReaction", b"mediaReaction", "originalTransportPayload", b"originalTransportPayload", "reaction", b"reaction", "supplementMessageContent", b"supplementMessageContent"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["supplementMessageContent", b"supplementMessageContent"]) -> typing.Literal["reaction", "contentView", "editText", "mediaReaction", "originalTransportPayload", "mediaInterventions"] | None: ...

global___SupplementMessageContent = SupplementMessageContent

@typing.final
class MediaReaction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIAID_FIELD_NUMBER: builtins.int
    REACTION_FIELD_NUMBER: builtins.int
    mediaID: builtins.str
    @property
    def reaction(self) -> global___Reaction: ...
    def __init__(
        self,
        *,
        mediaID: builtins.str | None = ...,
        reaction: global___Reaction | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["mediaID", b"mediaID", "reaction", b"reaction"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["mediaID", b"mediaID", "reaction", b"reaction"]) -> None: ...

global___MediaReaction = MediaReaction

@typing.final
class Reaction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REACTIONTYPE_FIELD_NUMBER: builtins.int
    REACTIONSTATUS_FIELD_NUMBER: builtins.int
    EMOJI_FIELD_NUMBER: builtins.int
    SUPERREACTTYPE_FIELD_NUMBER: builtins.int
    ACTIONLOGOTID_FIELD_NUMBER: builtins.int
    reactionType: builtins.str
    reactionStatus: builtins.str
    emoji: builtins.str
    superReactType: builtins.str
    actionLogOtid: builtins.str
    def __init__(
        self,
        *,
        reactionType: builtins.str | None = ...,
        reactionStatus: builtins.str | None = ...,
        emoji: builtins.str | None = ...,
        superReactType: builtins.str | None = ...,
        actionLogOtid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["actionLogOtid", b"actionLogOtid", "emoji", b"emoji", "reactionStatus", b"reactionStatus", "reactionType", b"reactionType", "superReactType", b"superReactType"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["actionLogOtid", b"actionLogOtid", "emoji", b"emoji", "reactionStatus", b"reactionStatus", "reactionType", b"reactionType", "superReactType", b"superReactType"]) -> None: ...

global___Reaction = Reaction

@typing.final
class ContentView(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SEEN_FIELD_NUMBER: builtins.int
    SCREENSHOTTED_FIELD_NUMBER: builtins.int
    REPLAYED_FIELD_NUMBER: builtins.int
    MIMETYPE_FIELD_NUMBER: builtins.int
    OBJECTID_FIELD_NUMBER: builtins.int
    seen: builtins.bool
    screenshotted: builtins.bool
    replayed: builtins.bool
    mimetype: builtins.str
    objectID: builtins.str
    def __init__(
        self,
        *,
        seen: builtins.bool | None = ...,
        screenshotted: builtins.bool | None = ...,
        replayed: builtins.bool | None = ...,
        mimetype: builtins.str | None = ...,
        objectID: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["mimetype", b"mimetype", "objectID", b"objectID", "replayed", b"replayed", "screenshotted", b"screenshotted", "seen", b"seen"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["mimetype", b"mimetype", "objectID", b"objectID", "replayed", b"replayed", "screenshotted", b"screenshotted", "seen", b"seen"]) -> None: ...

global___ContentView = ContentView

@typing.final
class EditText(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NEWCONTENT_FIELD_NUMBER: builtins.int
    EDITCOUNT_FIELD_NUMBER: builtins.int
    newContent: builtins.str
    editCount: builtins.int
    def __init__(
        self,
        *,
        newContent: builtins.str | None = ...,
        editCount: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["editCount", b"editCount", "newContent", b"newContent"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["editCount", b"editCount", "newContent", b"newContent"]) -> None: ...

global___EditText = EditText

@typing.final
class OriginalTransportPayload(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ORIGINALTRANSPORTPAYLOAD_FIELD_NUMBER: builtins.int
    originalTransportPayload: builtins.bytes
    def __init__(
        self,
        *,
        originalTransportPayload: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["originalTransportPayload", b"originalTransportPayload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["originalTransportPayload", b"originalTransportPayload"]) -> None: ...

global___OriginalTransportPayload = OriginalTransportPayload

@typing.final
class MediaInterventions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIAID_FIELD_NUMBER: builtins.int
    INTERVENTIONTYPE_FIELD_NUMBER: builtins.int
    mediaID: builtins.str
    interventionType: instamadilloCoreTypeMedia.InstamadilloCoreTypeMedia_pb2.Media.InterventionType.ValueType
    def __init__(
        self,
        *,
        mediaID: builtins.str | None = ...,
        interventionType: instamadilloCoreTypeMedia.InstamadilloCoreTypeMedia_pb2.Media.InterventionType.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["interventionType", b"interventionType", "mediaID", b"mediaID"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["interventionType", b"interventionType", "mediaID", b"mediaID"]) -> None: ...

global___MediaInterventions = MediaInterventions



from __future__ import annotations

import asyncio
import ctypes
from asyncio import Event as IOEvent
from typing import TYPE_CHECKING, Awaitable, Callable, Coroutine, Dict, Type, TypeVar

import segno
from google.protobuf.message import Message

from ..events import EVENT_TO_INT, INT_TO_EVENT, UnsupportedEvent, log
from ..proto.Neonize_pb2 import QR as QREv
from ..proto.Neonize_pb2 import BlocklistChange as BlocklistChangeEv
from ..proto.Neonize_pb2 import BlocklistEvent as BlocklistEv
from ..proto.Neonize_pb2 import CallAccept as CallAcceptEv
from ..proto.Neonize_pb2 import CallOffer as CallOfferEv
from ..proto.Neonize_pb2 import CallOfferNotice as CallOfferNoticeEv
from ..proto.Neonize_pb2 import CallPreAccept as CallPreAcceptEv
from ..proto.Neonize_pb2 import CallRelayLatency as CallRelayLatencyEv
from ..proto.Neonize_pb2 import CallTerminate as CallTerminateEv
from ..proto.Neonize_pb2 import CallTransport as CallTransportEv
from ..proto.Neonize_pb2 import ChatPresence as ChatPresenceEv
from ..proto.Neonize_pb2 import ClientOutdated as ClientOutdatedEv
from ..proto.Neonize_pb2 import Connected as ConnectedEv
from ..proto.Neonize_pb2 import ConnectFailure as ConnectFailureEv
from ..proto.Neonize_pb2 import Disconnected as DisconnectedEv
from ..proto.Neonize_pb2 import GroupInfoEvent as GroupInfoEv
from ..proto.Neonize_pb2 import HistorySync as HistorySyncEv
from ..proto.Neonize_pb2 import IdentityChange as IdentityChangeEv
from ..proto.Neonize_pb2 import JoinedGroup as JoinedGroupEv
from ..proto.Neonize_pb2 import KeepAliveRestored as KeepAliveRestoredEv
from ..proto.Neonize_pb2 import KeepAliveTimeout as KeepAliveTimeoutEv
from ..proto.Neonize_pb2 import LoggedOut as LoggedOutEv
from ..proto.Neonize_pb2 import Message as MessageEv
from ..proto.Neonize_pb2 import NewsletterJoin as NewsletterJoinEv
from ..proto.Neonize_pb2 import NewsletterLeave as NewsletterLeaveEv
from ..proto.Neonize_pb2 import NewsletterLiveUpdate as NewsletterLiveUpdateEv
from ..proto.Neonize_pb2 import NewsLetterMessageMeta as NewsLetterMessageMetaEv
from ..proto.Neonize_pb2 import NewsletterMuteChange as NewsletterMuteChangeEv
from ..proto.Neonize_pb2 import OfflineSyncCompleted as OfflineSyncCompletedEv
from ..proto.Neonize_pb2 import OfflineSyncPreview as OfflineSyncPreviewEv
from ..proto.Neonize_pb2 import PairStatus as PairStatusEv
from ..proto.Neonize_pb2 import Picture as PictureEv
from ..proto.Neonize_pb2 import Presence as PresenceEv
from ..proto.Neonize_pb2 import Receipt as ReceiptEv
from ..proto.Neonize_pb2 import StreamError as StreamErrorEv
from ..proto.Neonize_pb2 import StreamReplaced as StreamReplacedEv
from ..proto.Neonize_pb2 import TemporaryBan as TemporaryBanEv
from ..proto.Neonize_pb2 import UnknownCallEvent as UnknownCallEventEv
from ..proto.Neonize_pb2 import privacySettingsEvent as PrivacySettingsEv

event_global_loop = asyncio.new_event_loop()

if TYPE_CHECKING:
    from .client import ClientFactory, NewAClient
EventType = TypeVar("EventType", bound=Message)
event = IOEvent()
__all__ = [
    "QREv",
    "PairStatusEv",
    "ConnectedEv",
    "KeepAliveTimeoutEv",
    "KeepAliveRestoredEv",
    "LoggedOutEv",
    "StreamReplacedEv",
    "TemporaryBanEv",
    "ConnectFailureEv",
    "ClientOutdatedEv",
    "StreamErrorEv",
    "DisconnectedEv",
    "HistorySyncEv",
    "NewsLetterMessageMetaEv",
    "MessageEv",
    "ReceiptEv",
    "ChatPresenceEv",
    "PresenceEv",
    "JoinedGroupEv",
    "GroupInfoEv",
    "PictureEv",
    "IdentityChangeEv",
    "PrivacySettingsEv",
    "OfflineSyncPreviewEv",
    "OfflineSyncCompletedEv",
    "BlocklistEv",
    "BlocklistChangeEv",
    "NewsletterJoinEv",
    "NewsletterLeaveEv",
    "NewsletterMuteChangeEv",
    "NewsletterLiveUpdateEv",
    "CallOfferEv",
    "CallAcceptEv",
    "CallPreAcceptEv",
    "CallTransportEv",
    "CallOfferNoticeEv",
    "CallRelayLatencyEv",
    "CallTerminateEv",
    "UnknownCallEventEv",
]


class Event:
    def __init__(self, client: NewAClient):
        """
        Initializes the Event class with a client of type NewClient.
        Also sets up a default blocking function and an empty dictionary for list functions.

        :param client: An instance of the NewClient class
        :type client: NewClient
        """
        self.client = client
        self.blocking_func = self.paircode(self.default_paircode_cb)
        self.list_func: Dict[
            int, Callable[[NewAClient, Message], Coroutine[None, None, None]]
        ] = {}
        self._qr = self.__onqr

    def execute(self, uuid: int, binary: int, size: int, code: int):
        """Executes a function from the list of functions based on the given code.

        :param binary: The binary data to be processed by the function.
        :type binary: int
        :param size: The size of the binary data.
        :type size: int
        :param code: The index of the function to be executed from the list of functions.
        :type code: int
        """
        if code not in INT_TO_EVENT:
            raise UnsupportedEvent()
        # print("key", code, "uuid", uuid, "size", size)
        # print(f"Executing function for event code {ctypes.string_at(code, size)} with UUID {uuid}")
        message = INT_TO_EVENT[code].FromString(ctypes.string_at(binary, size))

        if code == 0:
            self.client.me = message
            return
        elif code == 3:
            self.client.connected = True
        # loop = asyncio.new_event_loop()
        # loop.run_until_complete(
        #     self.list_func[code](self.client, message)
        # )
        # loop.close()
        asyncio.run_coroutine_threadsafe(
            self.list_func[code](self.client, message), event_global_loop
        )

    async def __onqr(self, _: NewAClient, data_qr: bytes):
        """
        Handles QR code generation and display.

        :param _: The client instance (not used in the function).
        :type _: NewClient
        :param data_qr: The data to be encoded in the QR code.
        :type data_qr: bytes
        """
        segno.make_qr(data_qr).terminal(compact=True)

    def qr(self, f: Callable[[NewAClient, bytes], Coroutine[None, None, None]]):
        """
        Sets a callback function for handling QR code data.

        :param f: The callback function that takes a NewClient instance and QR code data in bytes.
        :type f: Callable[[NewClient, bytes], None]
        """
        self._qr = f

    @property
    def paircode(self):
        def paircodecb(
            f: Callable[[NewAClient, str, bool], Coroutine[None, None, None]],
        ):
            """
            A decorator that registers a callback function for handling pair code events.
            :param f: The callback function that takes a NewAClient instance, pair code as a string, and a boolean indicating if the connection is established.
            :type f: Callable[[NewAClient, str, bool], Coroutine[None, None, None]]
            :return: A function that wraps the callback function to handle pair code events.
            :rtype: Callable[[NewAClient, bytes, bool], Coroutine[None, None, None]]
            """

            def wrap_paircode(_, code, connected: bool):
                paircode = ctypes.string_at(code)
                asyncio.run_coroutine_threadsafe(
                    f(self.client, paircode.decode(), connected), event_global_loop
                ).result()

            self.blocking_func = wrap_paircode
            return self.blocking_func

        return paircodecb

    @staticmethod
    async def default_paircode_cb(
        client: NewAClient, data: str, connected: bool = True
    ):
        """
        A default callback function that handles the pair code event.
        This function is called when the pair code event occurs, and it blocks the execution until the event is processed.
        :param client: The client instance that triggered the event.
        :type client: NewAClient
        :param data: The pair code data as a string.
        :type data: str
        """
        if connected:
            log.info("Pair code successfully processed: %s", data)
        else:
            log.info("Pair code: %s", data)

    def __call__(
        self, event: Type[EventType]
    ) -> Callable[[Callable[[NewAClient, EventType], Awaitable[None]]], None]:
        """
        Registers a callback function for a specific event type.

        :param event: The type of event to register the callback for.
        :type event: Type[EventType]
        :return: A decorator that registers the callback function.
        :rtype: Callable[[Callable[[NewClient, EventType], None]], None]
        """

        def callback(func: Callable[[NewAClient, EventType], Awaitable[None]]) -> None:
            self.list_func.update({EVENT_TO_INT[event]: func})

        return callback


class EventsManager:
    def __init__(self, client_factory: ClientFactory):
        self.client_factory = client_factory
        self.list_func: Dict[int, Callable[[NewAClient, Message], Awaitable[None]]] = {}

    def __call__(
        self, event: Type[EventType]
    ) -> Callable[[Callable[[NewAClient, EventType], Awaitable[None]]], None]:
        """
        Registers a callback function for a specific event type.

        :param event: The type of event to register the callback for.
        :type event: Type[EventType]
        :return: A decorator that registers the callback function.
        :rtype: Callable[[Callable[[NewClient, EventType], None]], None]
        """

        def callback(func: Callable[[NewAClient, EventType], Awaitable[None]]) -> None:
            self.list_func.update({EVENT_TO_INT[event]: func})

        return callback


# threading.Thread(
#    target=event_global_loop.run_forever,
#    daemon=True,
# ).start()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: waStatusAttributions/WAWebProtobufsStatusAttributions.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;waStatusAttributions/WAWebProtobufsStatusAttributions.proto\x12 WAWebProtobufsStatusAttributions\"\xda\n\n\x11StatusAttribution\x12Z\n\rstatusReshare\x18\x03 \x01(\x0b\x32\x41.WAWebProtobufsStatusAttributions.StatusAttribution.StatusReshareH\x00\x12Z\n\rexternalShare\x18\x04 \x01(\x0b\x32\x41.WAWebProtobufsStatusAttributions.StatusAttribution.ExternalShareH\x00\x12J\n\x05music\x18\x05 \x01(\x0b\x32\x39.WAWebProtobufsStatusAttributions.StatusAttribution.MusicH\x00\x12V\n\x0bgroupStatus\x18\x06 \x01(\x0b\x32?.WAWebProtobufsStatusAttributions.StatusAttribution.GroupStatusH\x00\x12\x46\n\x04type\x18\x01 \x01(\x0e\x32\x38.WAWebProtobufsStatusAttributions.StatusAttribution.Type\x12\x11\n\tactionURL\x18\x02 \x01(\t\x1a\xf9\x01\n\rExternalShare\x12\x11\n\tactionURL\x18\x01 \x01(\t\x12X\n\x06source\x18\x02 \x01(\x0e\x32H.WAWebProtobufsStatusAttributions.StatusAttribution.ExternalShare.Source\x12\x10\n\x08\x64uration\x18\x03 \x01(\x05\x12\x19\n\x11\x61\x63tionFallbackURL\x18\x04 \x01(\t\"N\n\x06Source\x12\x0b\n\x07UNKNOWN\x10\x00\x12\r\n\tINSTAGRAM\x10\x01\x12\x0c\n\x08\x46\x41\x43\x45\x42OOK\x10\x02\x12\r\n\tMESSENGER\x10\x03\x12\x0b\n\x07SPOTIFY\x10\x04\x1a\x87\x03\n\rStatusReshare\x12X\n\x06source\x18\x01 \x01(\x0e\x32H.WAWebProtobufsStatusAttributions.StatusAttribution.StatusReshare.Source\x12\\\n\x08metadata\x18\x02 \x01(\x0b\x32J.WAWebProtobufsStatusAttributions.StatusAttribution.StatusReshare.Metadata\x1ag\n\x08Metadata\x12\x10\n\x08\x64uration\x18\x01 \x01(\x05\x12\x12\n\nchannelJID\x18\x02 \x01(\t\x12\x18\n\x10\x63hannelMessageID\x18\x03 \x01(\x05\x12\x1b\n\x13hasMultipleReshares\x18\x04 \x01(\x08\"U\n\x06Source\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x14\n\x10INTERNAL_RESHARE\x10\x01\x12\x13\n\x0fMENTION_RESHARE\x10\x02\x12\x13\n\x0f\x43HANNEL_RESHARE\x10\x03\x1a \n\x0bGroupStatus\x12\x11\n\tauthorJID\x18\x01 \x01(\t\x1ay\n\x05Music\x12\x12\n\nauthorName\x18\x01 \x01(\t\x12\x0e\n\x06songID\x18\x02 \x01(\t\x12\r\n\x05title\x18\x03 \x01(\t\x12\x0e\n\x06\x61uthor\x18\x04 \x01(\t\x12\x19\n\x11\x61rtistAttribution\x18\x05 \x01(\t\x12\x12\n\nisExplicit\x18\x06 \x01(\x08\"X\n\x04Type\x12\x0b\n\x07RESHARE\x10\x00\x12\x12\n\x0e\x45XTERNAL_SHARE\x10\x01\x12\t\n\x05MUSIC\x10\x02\x12\x12\n\x0eSTATUS_MENTION\x10\x03\x12\x10\n\x0cGROUP_STATUS\x10\x04\x42\x11\n\x0f\x61ttributionDataB0Z.go.mau.fi/whatsmeow/proto/waStatusAttributions')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'waStatusAttributions.WAWebProtobufsStatusAttributions_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z.go.mau.fi/whatsmeow/proto/waStatusAttributions'
  _globals['_STATUSATTRIBUTION']._serialized_start=98
  _globals['_STATUSATTRIBUTION']._serialized_end=1468
  _globals['_STATUSATTRIBUTION_EXTERNALSHARE']._serialized_start=559
  _globals['_STATUSATTRIBUTION_EXTERNALSHARE']._serialized_end=808
  _globals['_STATUSATTRIBUTION_EXTERNALSHARE_SOURCE']._serialized_start=730
  _globals['_STATUSATTRIBUTION_EXTERNALSHARE_SOURCE']._serialized_end=808
  _globals['_STATUSATTRIBUTION_STATUSRESHARE']._serialized_start=811
  _globals['_STATUSATTRIBUTION_STATUSRESHARE']._serialized_end=1202
  _globals['_STATUSATTRIBUTION_STATUSRESHARE_METADATA']._serialized_start=1012
  _globals['_STATUSATTRIBUTION_STATUSRESHARE_METADATA']._serialized_end=1115
  _globals['_STATUSATTRIBUTION_STATUSRESHARE_SOURCE']._serialized_start=1117
  _globals['_STATUSATTRIBUTION_STATUSRESHARE_SOURCE']._serialized_end=1202
  _globals['_STATUSATTRIBUTION_GROUPSTATUS']._serialized_start=1204
  _globals['_STATUSATTRIBUTION_GROUPSTATUS']._serialized_end=1236
  _globals['_STATUSATTRIBUTION_MUSIC']._serialized_start=1238
  _globals['_STATUSATTRIBUTION_MUSIC']._serialized_end=1359
  _globals['_STATUSATTRIBUTION_TYPE']._serialized_start=1361
  _globals['_STATUSATTRIBUTION_TYPE']._serialized_end=1449
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: instamadilloCoreTypeText/InstamadilloCoreTypeText.proto
# Protobuf Python Version: 6.32.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    32,
    1,
    '',
    'instamadilloCoreTypeText/InstamadilloCoreTypeText.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from instamadilloCoreTypeMedia import InstamadilloCoreTypeMedia_pb2 as instamadilloCoreTypeMedia_dot_InstamadilloCoreTypeMedia__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n7instamadilloCoreTypeText/InstamadilloCoreTypeText.proto\x12\x18InstamadilloCoreTypeText\x1a\x39instamadilloCoreTypeMedia/InstamadilloCoreTypeMedia.proto\"\xf1\x03\n\x04Text\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x18\n\x10isSuggestedReply\x18\x02 \x01(\x08\x12\x17\n\x0fpostbackPayload\x18\x03 \x01(\t\x12;\n\x0bpowerUpData\x18\x04 \x01(\x0b\x32&.InstamadilloCoreTypeText.PowerUpsData\x12<\n\x08\x63ommands\x18\x05 \x03(\x0b\x32*.InstamadilloCoreTypeText.CommandRangeData\x12[\n\x1c\x61nimatedEmojiCharacterRanges\x18\x06 \x03(\x0b\x32\x35.InstamadilloCoreTypeText.AnimatedEmojiCharacterRange\"\xcf\x01\n\x0b\x46ormatStyle\x12!\n\x1dTEXT_FORMAT_STYLE_UNSPECIFIED\x10\x00\x12\x1a\n\x16TEXT_FORMAT_STYLE_BOLD\x10\x01\x12\x1c\n\x18TEXT_FORMAT_STYLE_ITALIC\x10\x02\x12#\n\x1fTEXT_FORMAT_STYLE_STRIKETHROUGH\x10\x03\x12\x1f\n\x1bTEXT_FORMAT_STYLE_UNDERLINE\x10\x04\x12\x1d\n\x19TEXT_FORMAT_STYLE_INVALID\x10\x05\"g\n\x0cPowerUpsData\x12\r\n\x05style\x18\x01 \x01(\x05\x12H\n\x0fmediaAttachment\x18\x02 \x01(\x0b\x32/.InstamadilloCoreTypeMedia.CommonMediaTransport\"h\n\x10\x43ommandRangeData\x12\x0e\n\x06offset\x18\x01 \x01(\x05\x12\x0e\n\x06length\x18\x02 \x01(\x05\x12\x0c\n\x04type\x18\x03 \x01(\x05\x12\x0c\n\x04\x46\x42ID\x18\x04 \x01(\t\x12\x18\n\x10userOrThreadFbid\x18\x05 \x01(\t\"j\n\rFormattedText\x12\x0e\n\x06offset\x18\x01 \x01(\x05\x12\x0e\n\x06length\x18\x02 \x01(\x05\x12\x39\n\x05style\x18\x03 \x01(\x0e\x32*.InstamadilloCoreTypeText.Text.FormatStyle\"=\n\x1b\x41nimatedEmojiCharacterRange\x12\x0e\n\x06offset\x18\x01 \x01(\x05\x12\x0e\n\x06length\x18\x02 \x01(\x05\x42\x34Z2go.mau.fi/whatsmeow/proto/instamadilloCoreTypeText')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'instamadilloCoreTypeText.InstamadilloCoreTypeText_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z2go.mau.fi/whatsmeow/proto/instamadilloCoreTypeText'
  _globals['_TEXT']._serialized_start=145
  _globals['_TEXT']._serialized_end=642
  _globals['_TEXT_FORMATSTYLE']._serialized_start=435
  _globals['_TEXT_FORMATSTYLE']._serialized_end=642
  _globals['_POWERUPSDATA']._serialized_start=644
  _globals['_POWERUPSDATA']._serialized_end=747
  _globals['_COMMANDRANGEDATA']._serialized_start=749
  _globals['_COMMANDRANGEDATA']._serialized_end=853
  _globals['_FORMATTEDTEXT']._serialized_start=855
  _globals['_FORMATTEDTEXT']._serialized_end=961
  _globals['_ANIMATEDEMOJICHARACTERRANGE']._serialized_start=963
  _globals['_ANIMATEDEMOJICHARACTERRANGE']._serialized_end=1024
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _XmaActionType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _XmaActionTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_XmaActionType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    XMA_ACTION_TYPE_UNSPECIFIED: _XmaActionType.ValueType  # 0
    XMA_ACTION_TYPE_SHARE: _XmaActionType.ValueType  # 1
    XMA_ACTION_TYPE_REPLY: _XmaActionType.ValueType  # 2
    XMA_ACTION_TYPE_REACT: _XmaActionType.ValueType  # 3
    XMA_ACTION_TYPE_MENTION: _XmaActionType.ValueType  # 4

class XmaActionType(_XmaActionType, metaclass=_XmaActionTypeEnumTypeWrapper): ...

XMA_ACTION_TYPE_UNSPECIFIED: XmaActionType.ValueType  # 0
XMA_ACTION_TYPE_SHARE: XmaActionType.ValueType  # 1
XMA_ACTION_TYPE_REPLY: XmaActionType.ValueType  # 2
XMA_ACTION_TYPE_REACT: XmaActionType.ValueType  # 3
XMA_ACTION_TYPE_MENTION: XmaActionType.ValueType  # 4
global___XmaActionType = XmaActionType

class _ReceiverFetchContentType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ReceiverFetchContentTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ReceiverFetchContentType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    RECEIVER_FETCH_CONTENT_TYPE_UNSPECIFIED: _ReceiverFetchContentType.ValueType  # 0
    RECEIVER_FETCH_CONTENT_TYPE_NOTE: _ReceiverFetchContentType.ValueType  # 1
    RECEIVER_FETCH_CONTENT_TYPE_STORY: _ReceiverFetchContentType.ValueType  # 2
    RECEIVER_FETCH_CONTENT_TYPE_PROFILE: _ReceiverFetchContentType.ValueType  # 3
    RECEIVER_FETCH_CONTENT_TYPE_CLIP: _ReceiverFetchContentType.ValueType  # 4
    RECEIVER_FETCH_CONTENT_TYPE_FEED: _ReceiverFetchContentType.ValueType  # 5
    RECEIVER_FETCH_CONTENT_TYPE_LIVE: _ReceiverFetchContentType.ValueType  # 6
    RECEIVER_FETCH_CONTENT_TYPE_COMMENT: _ReceiverFetchContentType.ValueType  # 7
    RECEIVER_FETCH_CONTENT_TYPE_LOCATION_SHARE: _ReceiverFetchContentType.ValueType  # 8
    RECEIVER_FETCH_CONTENT_TYPE_REELS_AUDIO: _ReceiverFetchContentType.ValueType  # 9
    RECEIVER_FETCH_CONTENT_TYPE_MEDIA_NOTE: _ReceiverFetchContentType.ValueType  # 10
    RECEIVER_FETCH_CONTENT_TYPE_STORY_HIGHLIGHT: _ReceiverFetchContentType.ValueType  # 11
    RECEIVER_FETCH_CONTENT_TYPE_SOCIAL_CONTEXT: _ReceiverFetchContentType.ValueType  # 12

class ReceiverFetchContentType(_ReceiverFetchContentType, metaclass=_ReceiverFetchContentTypeEnumTypeWrapper): ...

RECEIVER_FETCH_CONTENT_TYPE_UNSPECIFIED: ReceiverFetchContentType.ValueType  # 0
RECEIVER_FETCH_CONTENT_TYPE_NOTE: ReceiverFetchContentType.ValueType  # 1
RECEIVER_FETCH_CONTENT_TYPE_STORY: ReceiverFetchContentType.ValueType  # 2
RECEIVER_FETCH_CONTENT_TYPE_PROFILE: ReceiverFetchContentType.ValueType  # 3
RECEIVER_FETCH_CONTENT_TYPE_CLIP: ReceiverFetchContentType.ValueType  # 4
RECEIVER_FETCH_CONTENT_TYPE_FEED: ReceiverFetchContentType.ValueType  # 5
RECEIVER_FETCH_CONTENT_TYPE_LIVE: ReceiverFetchContentType.ValueType  # 6
RECEIVER_FETCH_CONTENT_TYPE_COMMENT: ReceiverFetchContentType.ValueType  # 7
RECEIVER_FETCH_CONTENT_TYPE_LOCATION_SHARE: ReceiverFetchContentType.ValueType  # 8
RECEIVER_FETCH_CONTENT_TYPE_REELS_AUDIO: ReceiverFetchContentType.ValueType  # 9
RECEIVER_FETCH_CONTENT_TYPE_MEDIA_NOTE: ReceiverFetchContentType.ValueType  # 10
RECEIVER_FETCH_CONTENT_TYPE_STORY_HIGHLIGHT: ReceiverFetchContentType.ValueType  # 11
RECEIVER_FETCH_CONTENT_TYPE_SOCIAL_CONTEXT: ReceiverFetchContentType.ValueType  # 12
global___ReceiverFetchContentType = ReceiverFetchContentType

class _MediaNoteFetchParamsMessageType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _MediaNoteFetchParamsMessageTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_MediaNoteFetchParamsMessageType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    MEDIA_NOTE_FETCH_PARAMS_MESSAGE_TYPE_UNSPECIFIED: _MediaNoteFetchParamsMessageType.ValueType  # 0
    MEDIA_NOTE_FETCH_PARAMS_MESSAGE_TYPE_MENTION: _MediaNoteFetchParamsMessageType.ValueType  # 1
    MEDIA_NOTE_FETCH_PARAMS_MESSAGE_TYPE_REPLY: _MediaNoteFetchParamsMessageType.ValueType  # 2

class MediaNoteFetchParamsMessageType(_MediaNoteFetchParamsMessageType, metaclass=_MediaNoteFetchParamsMessageTypeEnumTypeWrapper): ...

MEDIA_NOTE_FETCH_PARAMS_MESSAGE_TYPE_UNSPECIFIED: MediaNoteFetchParamsMessageType.ValueType  # 0
MEDIA_NOTE_FETCH_PARAMS_MESSAGE_TYPE_MENTION: MediaNoteFetchParamsMessageType.ValueType  # 1
MEDIA_NOTE_FETCH_PARAMS_MESSAGE_TYPE_REPLY: MediaNoteFetchParamsMessageType.ValueType  # 2
global___MediaNoteFetchParamsMessageType = MediaNoteFetchParamsMessageType

@typing.final
class XmaContentRef(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACTIONTYPE_FIELD_NUMBER: builtins.int
    CONTENTTYPE_FIELD_NUMBER: builtins.int
    TARGETURL_FIELD_NUMBER: builtins.int
    USERNAME_FIELD_NUMBER: builtins.int
    OWNERFBID_FIELD_NUMBER: builtins.int
    FETCHPARAMS_FIELD_NUMBER: builtins.int
    actionType: global___XmaActionType.ValueType
    contentType: global___ReceiverFetchContentType.ValueType
    targetURL: builtins.str
    userName: builtins.str
    ownerFbid: builtins.str
    @property
    def fetchParams(self) -> global___ReceiverFetchXmaFetchParams: ...
    def __init__(
        self,
        *,
        actionType: global___XmaActionType.ValueType | None = ...,
        contentType: global___ReceiverFetchContentType.ValueType | None = ...,
        targetURL: builtins.str | None = ...,
        userName: builtins.str | None = ...,
        ownerFbid: builtins.str | None = ...,
        fetchParams: global___ReceiverFetchXmaFetchParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["actionType", b"actionType", "contentType", b"contentType", "fetchParams", b"fetchParams", "ownerFbid", b"ownerFbid", "targetURL", b"targetURL", "userName", b"userName"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["actionType", b"actionType", "contentType", b"contentType", "fetchParams", b"fetchParams", "ownerFbid", b"ownerFbid", "targetURL", b"targetURL", "userName", b"userName"]) -> None: ...

global___XmaContentRef = XmaContentRef

@typing.final
class ReceiverFetchXmaFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NOTEFETCHPARAMS_FIELD_NUMBER: builtins.int
    STORYFETCHPARAMS_FIELD_NUMBER: builtins.int
    PROFILEFETCHPARAMS_FIELD_NUMBER: builtins.int
    CLIPFETCHPARAMS_FIELD_NUMBER: builtins.int
    FEEDFETCHPARAMS_FIELD_NUMBER: builtins.int
    LIVEFETCHPARAMS_FIELD_NUMBER: builtins.int
    COMMENTFETCHPARAMS_FIELD_NUMBER: builtins.int
    LOCATIONSHAREFETCHPARAMS_FIELD_NUMBER: builtins.int
    REELSAUDIOFETCHPARAMS_FIELD_NUMBER: builtins.int
    MEDIANOTEFETCHPARAMS_FIELD_NUMBER: builtins.int
    SOCIALCONTEXTFETCHPARAMS_FIELD_NUMBER: builtins.int
    @property
    def noteFetchParams(self) -> global___ReceiverFetchXmaNoteFetchParams: ...
    @property
    def storyFetchParams(self) -> global___ReceiverFetchXmaStoryFetchParams: ...
    @property
    def profileFetchParams(self) -> global___ReceiverFetchXmaProfileFetchParams: ...
    @property
    def clipFetchParams(self) -> global___ReceiverFetchXmaClipFetchParams: ...
    @property
    def feedFetchParams(self) -> global___ReceiverFetchXmaFeedFetchParams: ...
    @property
    def liveFetchParams(self) -> global___ReceiverFetchXmaLiveFetchParams: ...
    @property
    def commentFetchParams(self) -> global___ReceiverFetchXmaCommentFetchParams: ...
    @property
    def locationShareFetchParams(self) -> global___ReceiverFetchXmaLocationShareFetchParams: ...
    @property
    def reelsAudioFetchParams(self) -> global___ReceiverFetchXmaReelsAudioFetchParams: ...
    @property
    def mediaNoteFetchParams(self) -> global___ReceiverFetchXmaMediaNoteFetchParams: ...
    @property
    def socialContextFetchParams(self) -> global___ReceiverFetchXmaSocialContextFetchParams: ...
    def __init__(
        self,
        *,
        noteFetchParams: global___ReceiverFetchXmaNoteFetchParams | None = ...,
        storyFetchParams: global___ReceiverFetchXmaStoryFetchParams | None = ...,
        profileFetchParams: global___ReceiverFetchXmaProfileFetchParams | None = ...,
        clipFetchParams: global___ReceiverFetchXmaClipFetchParams | None = ...,
        feedFetchParams: global___ReceiverFetchXmaFeedFetchParams | None = ...,
        liveFetchParams: global___ReceiverFetchXmaLiveFetchParams | None = ...,
        commentFetchParams: global___ReceiverFetchXmaCommentFetchParams | None = ...,
        locationShareFetchParams: global___ReceiverFetchXmaLocationShareFetchParams | None = ...,
        reelsAudioFetchParams: global___ReceiverFetchXmaReelsAudioFetchParams | None = ...,
        mediaNoteFetchParams: global___ReceiverFetchXmaMediaNoteFetchParams | None = ...,
        socialContextFetchParams: global___ReceiverFetchXmaSocialContextFetchParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["clipFetchParams", b"clipFetchParams", "commentFetchParams", b"commentFetchParams", "feedFetchParams", b"feedFetchParams", "liveFetchParams", b"liveFetchParams", "locationShareFetchParams", b"locationShareFetchParams", "mediaNoteFetchParams", b"mediaNoteFetchParams", "noteFetchParams", b"noteFetchParams", "profileFetchParams", b"profileFetchParams", "receiverFetchXmaFetchParams", b"receiverFetchXmaFetchParams", "reelsAudioFetchParams", b"reelsAudioFetchParams", "socialContextFetchParams", b"socialContextFetchParams", "storyFetchParams", b"storyFetchParams"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["clipFetchParams", b"clipFetchParams", "commentFetchParams", b"commentFetchParams", "feedFetchParams", b"feedFetchParams", "liveFetchParams", b"liveFetchParams", "locationShareFetchParams", b"locationShareFetchParams", "mediaNoteFetchParams", b"mediaNoteFetchParams", "noteFetchParams", b"noteFetchParams", "profileFetchParams", b"profileFetchParams", "receiverFetchXmaFetchParams", b"receiverFetchXmaFetchParams", "reelsAudioFetchParams", b"reelsAudioFetchParams", "socialContextFetchParams", b"socialContextFetchParams", "storyFetchParams", b"storyFetchParams"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["receiverFetchXmaFetchParams", b"receiverFetchXmaFetchParams"]) -> typing.Literal["noteFetchParams", "storyFetchParams", "profileFetchParams", "clipFetchParams", "feedFetchParams", "liveFetchParams", "commentFetchParams", "locationShareFetchParams", "reelsAudioFetchParams", "mediaNoteFetchParams", "socialContextFetchParams"] | None: ...

global___ReceiverFetchXmaFetchParams = ReceiverFetchXmaFetchParams

@typing.final
class ReceiverFetchXmaNoteFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NOTEIGID_FIELD_NUMBER: builtins.int
    noteIgid: builtins.str
    def __init__(
        self,
        *,
        noteIgid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["noteIgid", b"noteIgid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["noteIgid", b"noteIgid"]) -> None: ...

global___ReceiverFetchXmaNoteFetchParams = ReceiverFetchXmaNoteFetchParams

@typing.final
class ReceiverFetchXmaStoryFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STORYIGID_FIELD_NUMBER: builtins.int
    REELID_FIELD_NUMBER: builtins.int
    storyIgid: builtins.str
    reelID: builtins.str
    def __init__(
        self,
        *,
        storyIgid: builtins.str | None = ...,
        reelID: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["reelID", b"reelID", "storyIgid", b"storyIgid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["reelID", b"reelID", "storyIgid", b"storyIgid"]) -> None: ...

global___ReceiverFetchXmaStoryFetchParams = ReceiverFetchXmaStoryFetchParams

@typing.final
class ReceiverFetchXmaProfileFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROFILEIGID_FIELD_NUMBER: builtins.int
    profileIgid: builtins.str
    def __init__(
        self,
        *,
        profileIgid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["profileIgid", b"profileIgid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["profileIgid", b"profileIgid"]) -> None: ...

global___ReceiverFetchXmaProfileFetchParams = ReceiverFetchXmaProfileFetchParams

@typing.final
class ReceiverFetchXmaClipFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIAIGID_FIELD_NUMBER: builtins.int
    mediaIgid: builtins.str
    def __init__(
        self,
        *,
        mediaIgid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["mediaIgid", b"mediaIgid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["mediaIgid", b"mediaIgid"]) -> None: ...

global___ReceiverFetchXmaClipFetchParams = ReceiverFetchXmaClipFetchParams

@typing.final
class ReceiverFetchXmaFeedFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIAIGID_FIELD_NUMBER: builtins.int
    CAROUSELSHARECHILDMEDIAIGID_FIELD_NUMBER: builtins.int
    mediaIgid: builtins.str
    carouselShareChildMediaIgid: builtins.str
    def __init__(
        self,
        *,
        mediaIgid: builtins.str | None = ...,
        carouselShareChildMediaIgid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["carouselShareChildMediaIgid", b"carouselShareChildMediaIgid", "mediaIgid", b"mediaIgid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["carouselShareChildMediaIgid", b"carouselShareChildMediaIgid", "mediaIgid", b"mediaIgid"]) -> None: ...

global___ReceiverFetchXmaFeedFetchParams = ReceiverFetchXmaFeedFetchParams

@typing.final
class ReceiverFetchXmaLiveFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LIVEIGID_FIELD_NUMBER: builtins.int
    liveIgid: builtins.str
    def __init__(
        self,
        *,
        liveIgid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["liveIgid", b"liveIgid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["liveIgid", b"liveIgid"]) -> None: ...

global___ReceiverFetchXmaLiveFetchParams = ReceiverFetchXmaLiveFetchParams

@typing.final
class ReceiverFetchXmaCommentFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMENTFBID_FIELD_NUMBER: builtins.int
    commentFbid: builtins.str
    def __init__(
        self,
        *,
        commentFbid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["commentFbid", b"commentFbid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["commentFbid", b"commentFbid"]) -> None: ...

global___ReceiverFetchXmaCommentFetchParams = ReceiverFetchXmaCommentFetchParams

@typing.final
class ReceiverFetchXmaLocationShareFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LOCATIONIGID_FIELD_NUMBER: builtins.int
    locationIgid: builtins.str
    def __init__(
        self,
        *,
        locationIgid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["locationIgid", b"locationIgid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["locationIgid", b"locationIgid"]) -> None: ...

global___ReceiverFetchXmaLocationShareFetchParams = ReceiverFetchXmaLocationShareFetchParams

@typing.final
class ReceiverFetchXmaReelsAudioFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUDIOIGID_FIELD_NUMBER: builtins.int
    audioIgid: builtins.str
    def __init__(
        self,
        *,
        audioIgid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["audioIgid", b"audioIgid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["audioIgid", b"audioIgid"]) -> None: ...

global___ReceiverFetchXmaReelsAudioFetchParams = ReceiverFetchXmaReelsAudioFetchParams

@typing.final
class ReceiverFetchXmaMediaNoteFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIANOTEIGID_FIELD_NUMBER: builtins.int
    MESSAGETYPE_FIELD_NUMBER: builtins.int
    mediaNoteIgid: builtins.str
    messageType: global___MediaNoteFetchParamsMessageType.ValueType
    def __init__(
        self,
        *,
        mediaNoteIgid: builtins.str | None = ...,
        messageType: global___MediaNoteFetchParamsMessageType.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["mediaNoteIgid", b"mediaNoteIgid", "messageType", b"messageType"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["mediaNoteIgid", b"mediaNoteIgid", "messageType", b"messageType"]) -> None: ...

global___ReceiverFetchXmaMediaNoteFetchParams = ReceiverFetchXmaMediaNoteFetchParams

@typing.final
class ReceiverFetchXmaSocialContextFetchParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MEDIAIGID_FIELD_NUMBER: builtins.int
    mediaIgid: builtins.str
    def __init__(
        self,
        *,
        mediaIgid: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["mediaIgid", b"mediaIgid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["mediaIgid", b"mediaIgid"]) -> None: ...

global___ReceiverFetchXmaSocialContextFetchParams = ReceiverFetchXmaSocialContextFetchParams



import ctypes
import logging
import queue
import threading

from ..proto.Neonize_pb2 import LogEntry

try:
    from colorlog import ColoredFormatter
except Exception:
    ColoredFormatter = None

log = logging.getLogger(__name__)
_log_ = log

if ColoredFormatter:
    formatter = ColoredFormatter(
        "%(asctime)s.%(msecs)03d %(log_color)s[%(name)s %(levelname)s] - %(message)s%(reset)s",
        datefmt="%H:%M:%S",
        log_colors={
            "INFO": "cyan",
            "WARNING": "yellow",
            "ERROR": "red",
            "CRITICAL": "bold_red",
        },
    )
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(formatter)
else:
    stream_handler = logging.StreamHandler()

logging.basicConfig(
    format="%(asctime)s.%(msecs)03d [%(name)s %(levelname)s] - %(message)s",
    datefmt="%H:%M:%S",
    level=logging.INFO,
    handlers=[stream_handler],
)

clientlogger = logging.getLogger("whatsmeow.Client")
dblogger = logging.getLogger("Whatsmeow.Database")


_log_queue = queue.Queue()


def _worker():
    while True:
        binary, size = _log_queue.get()
        if binary is None:
            break  # sentinel for shutdown
        try:
            log_msg = LogEntry.FromString(ctypes.string_at(binary, size))
            if log_msg.Name == "Client":
                log = clientlogger
            elif log_msg.Name == "Database":
                log = dblogger
            else:
                name = log_msg.Name.replace("/", ".")
                log = logging.getLogger(f"whatsmeow.{name}")
            level_fn = getattr(log, log_msg.Level.lower(), log.info)
            level_fn(log_msg.Message)
        except Exception:
            _log_.exception("Failed to handle WhatsMeow log")
        finally:
            _log_queue.task_done()


_thread = threading.Thread(target=_worker, daemon=True)
_thread.start()


def log_whatsmeow(binary: int, size: int):
    _log_queue.put((binary, size))


def shutdown_log_worker():
    _log_queue.put((None, 0))
    _thread.join()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import instamadilloCoreTypeActionLog.InstamadilloCoreTypeActionLog_pb2
import instamadilloCoreTypeAdminMessage.InstamadilloCoreTypeAdminMessage_pb2
import instamadilloCoreTypeCollection.InstamadilloCoreTypeCollection_pb2
import instamadilloCoreTypeLink.InstamadilloCoreTypeLink_pb2
import instamadilloCoreTypeMedia.InstamadilloCoreTypeMedia_pb2
import instamadilloCoreTypeText.InstamadilloCoreTypeText_pb2
import instamadilloXmaContentRef.InstamadilloXmaContentRef_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class AddMessagePayload(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENT_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    @property
    def content(self) -> global___AddMessageContent: ...
    @property
    def metadata(self) -> global___AddMessageMetadata: ...
    def __init__(
        self,
        *,
        content: global___AddMessageContent | None = ...,
        metadata: global___AddMessageMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["content", b"content", "metadata", b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["content", b"content", "metadata", b"metadata"]) -> None: ...

global___AddMessagePayload = AddMessagePayload

@typing.final
class AddMessageContent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TEXT_FIELD_NUMBER: builtins.int
    LIKE_FIELD_NUMBER: builtins.int
    LINK_FIELD_NUMBER: builtins.int
    RECEIVERFETCHXMA_FIELD_NUMBER: builtins.int
    MEDIA_FIELD_NUMBER: builtins.int
    PLACEHOLDER_FIELD_NUMBER: builtins.int
    COLLECTION_FIELD_NUMBER: builtins.int
    ADMINMESSAGE_FIELD_NUMBER: builtins.int
    ACTIONLOG_FIELD_NUMBER: builtins.int
    @property
    def text(self) -> instamadilloCoreTypeText.InstamadilloCoreTypeText_pb2.Text: ...
    @property
    def like(self) -> global___Like: ...
    @property
    def link(self) -> instamadilloCoreTypeLink.InstamadilloCoreTypeLink_pb2.Link: ...
    @property
    def receiverFetchXma(self) -> global___ReceiverFetchXma: ...
    @property
    def media(self) -> instamadilloCoreTypeMedia.InstamadilloCoreTypeMedia_pb2.Media: ...
    @property
    def placeholder(self) -> global___Placeholder: ...
    @property
    def collection(self) -> instamadilloCoreTypeCollection.InstamadilloCoreTypeCollection_pb2.Collection: ...
    @property
    def adminMessage(self) -> instamadilloCoreTypeAdminMessage.InstamadilloCoreTypeAdminMessage_pb2.AdminMessage: ...
    @property
    def actionLog(self) -> instamadilloCoreTypeActionLog.InstamadilloCoreTypeActionLog_pb2.ActionLog: ...
    def __init__(
        self,
        *,
        text: instamadilloCoreTypeText.InstamadilloCoreTypeText_pb2.Text | None = ...,
        like: global___Like | None = ...,
        link: instamadilloCoreTypeLink.InstamadilloCoreTypeLink_pb2.Link | None = ...,
        receiverFetchXma: global___ReceiverFetchXma | None = ...,
        media: instamadilloCoreTypeMedia.InstamadilloCoreTypeMedia_pb2.Media | None = ...,
        placeholder: global___Placeholder | None = ...,
        collection: instamadilloCoreTypeCollection.InstamadilloCoreTypeCollection_pb2.Collection | None = ...,
        adminMessage: instamadilloCoreTypeAdminMessage.InstamadilloCoreTypeAdminMessage_pb2.AdminMessage | None = ...,
        actionLog: instamadilloCoreTypeActionLog.InstamadilloCoreTypeActionLog_pb2.ActionLog | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["actionLog", b"actionLog", "addMessageContent", b"addMessageContent", "adminMessage", b"adminMessage", "collection", b"collection", "like", b"like", "link", b"link", "media", b"media", "placeholder", b"placeholder", "receiverFetchXma", b"receiverFetchXma", "text", b"text"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["actionLog", b"actionLog", "addMessageContent", b"addMessageContent", "adminMessage", b"adminMessage", "collection", b"collection", "like", b"like", "link", b"link", "media", b"media", "placeholder", b"placeholder", "receiverFetchXma", b"receiverFetchXma", "text", b"text"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["addMessageContent", b"addMessageContent"]) -> typing.Literal["text", "like", "link", "receiverFetchXma", "media", "placeholder", "collection", "adminMessage", "actionLog"] | None: ...

global___AddMessageContent = AddMessageContent

@typing.final
class AddMessageMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SENDSILENTLY_FIELD_NUMBER: builtins.int
    PRIVATEREPLYINFO_FIELD_NUMBER: builtins.int
    REPLIEDTOMESSAGE_FIELD_NUMBER: builtins.int
    FORWARDINGPARAMS_FIELD_NUMBER: builtins.int
    EPHEMERALITYPARAMS_FIELD_NUMBER: builtins.int
    sendSilently: builtins.bool
    @property
    def privateReplyInfo(self) -> global___PrivateReplyInfo: ...
    @property
    def repliedToMessage(self) -> global___RepliedToMessage: ...
    @property
    def forwardingParams(self) -> global___ForwardingParams: ...
    @property
    def ephemeralityParams(self) -> global___EphemeralityParams: ...
    def __init__(
        self,
        *,
        sendSilently: builtins.bool | None = ...,
        privateReplyInfo: global___PrivateReplyInfo | None = ...,
        repliedToMessage: global___RepliedToMessage | None = ...,
        forwardingParams: global___ForwardingParams | None = ...,
        ephemeralityParams: global___EphemeralityParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["ephemeralityParams", b"ephemeralityParams", "forwardingParams", b"forwardingParams", "privateReplyInfo", b"privateReplyInfo", "repliedToMessage", b"repliedToMessage", "sendSilently", b"sendSilently"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["ephemeralityParams", b"ephemeralityParams", "forwardingParams", b"forwardingParams", "privateReplyInfo", b"privateReplyInfo", "repliedToMessage", b"repliedToMessage", "sendSilently", b"sendSilently"]) -> None: ...

global___AddMessageMetadata = AddMessageMetadata

@typing.final
class RepliedToMessage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REPLIEDTOMESSAGEOTID_FIELD_NUMBER: builtins.int
    REPLIEDTOMESSAGEWASERVERTIMESEC_FIELD_NUMBER: builtins.int
    REPLIEDTOMESSAGECOLLECTIONITEMID_FIELD_NUMBER: builtins.int
    OMMICROSECTS_FIELD_NUMBER: builtins.int
    repliedToMessageOtid: builtins.str
    repliedToMessageWaServerTimeSec: builtins.str
    repliedToMessageCollectionItemID: builtins.str
    @property
    def omMicroSecTS(self) -> global___OpenMessageMicroSecondTimestamp: ...
    def __init__(
        self,
        *,
        repliedToMessageOtid: builtins.str | None = ...,
        repliedToMessageWaServerTimeSec: builtins.str | None = ...,
        repliedToMessageCollectionItemID: builtins.str | None = ...,
        omMicroSecTS: global___OpenMessageMicroSecondTimestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["omMicroSecTS", b"omMicroSecTS", "repliedToMessageCollectionItemID", b"repliedToMessageCollectionItemID", "repliedToMessageOtid", b"repliedToMessageOtid", "repliedToMessageWaServerTimeSec", b"repliedToMessageWaServerTimeSec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["omMicroSecTS", b"omMicroSecTS", "repliedToMessageCollectionItemID", b"repliedToMessageCollectionItemID", "repliedToMessageOtid", b"repliedToMessageOtid", "repliedToMessageWaServerTimeSec", b"repliedToMessageWaServerTimeSec"]) -> None: ...

global___RepliedToMessage = RepliedToMessage

@typing.final
class OpenMessageMicroSecondTimestamp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMESTAMPMS_FIELD_NUMBER: builtins.int
    MICROSECONDSBITS_FIELD_NUMBER: builtins.int
    timestampMS: builtins.int
    microSecondsBits: builtins.int
    def __init__(
        self,
        *,
        timestampMS: builtins.int | None = ...,
        microSecondsBits: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["microSecondsBits", b"microSecondsBits", "timestampMS", b"timestampMS"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["microSecondsBits", b"microSecondsBits", "timestampMS", b"timestampMS"]) -> None: ...

global___OpenMessageMicroSecondTimestamp = OpenMessageMicroSecondTimestamp

@typing.final
class PrivateReplyInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMENTID_FIELD_NUMBER: builtins.int
    POSTLINK_FIELD_NUMBER: builtins.int
    commentID: builtins.str
    postLink: builtins.str
    def __init__(
        self,
        *,
        commentID: builtins.str | None = ...,
        postLink: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["commentID", b"commentID", "postLink", b"postLink"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["commentID", b"commentID", "postLink", b"postLink"]) -> None: ...

global___PrivateReplyInfo = PrivateReplyInfo

@typing.final
class ForwardingParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FORWARDEDTHREADID_FIELD_NUMBER: builtins.int
    forwardedThreadID: builtins.str
    def __init__(
        self,
        *,
        forwardedThreadID: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["forwardedThreadID", b"forwardedThreadID"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["forwardedThreadID", b"forwardedThreadID"]) -> None: ...

global___ForwardingParams = ForwardingParams

@typing.final
class EphemeralityParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EPHEMERALDURATIONSEC_FIELD_NUMBER: builtins.int
    ephemeralDurationSec: builtins.int
    def __init__(
        self,
        *,
        ephemeralDurationSec: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["ephemeralDurationSec", b"ephemeralDurationSec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["ephemeralDurationSec", b"ephemeralDurationSec"]) -> None: ...

global___EphemeralityParams = EphemeralityParams

@typing.final
class Like(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Like = Like

@typing.final
class ReceiverFetchXma(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENTREF_FIELD_NUMBER: builtins.int
    TEXT_FIELD_NUMBER: builtins.int
    MEDIA_FIELD_NUMBER: builtins.int
    XMACONTENTREF_FIELD_NUMBER: builtins.int
    contentRef: builtins.str
    text: builtins.str
    @property
    def media(self) -> instamadilloCoreTypeMedia.InstamadilloCoreTypeMedia_pb2.Media: ...
    @property
    def xmaContentRef(self) -> instamadilloXmaContentRef.InstamadilloXmaContentRef_pb2.XmaContentRef: ...
    def __init__(
        self,
        *,
        contentRef: builtins.str | None = ...,
        text: builtins.str | None = ...,
        media: instamadilloCoreTypeMedia.InstamadilloCoreTypeMedia_pb2.Media | None = ...,
        xmaContentRef: instamadilloXmaContentRef.InstamadilloXmaContentRef_pb2.XmaContentRef | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["contentRef", b"contentRef", "media", b"media", "text", b"text", "xmaContentRef", b"xmaContentRef"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["contentRef", b"contentRef", "media", b"media", "text", b"text", "xmaContentRef", b"xmaContentRef"]) -> None: ...

global___ReceiverFetchXma = ReceiverFetchXma

@typing.final
class Placeholder(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Type:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Placeholder._Type.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        PLACEHOLDER_TYPE_NONE: Placeholder._Type.ValueType  # 0
        PLACEHOLDER_TYPE_DECRYPTION_FAILURE: Placeholder._Type.ValueType  # 1
        PLACEHOLDER_TYPE_NOT_SUPPORTED_NEED_UPDATE: Placeholder._Type.ValueType  # 2
        PLACEHOLDER_TYPE_DEVICE_UNAVAILABLE: Placeholder._Type.ValueType  # 3
        PLACEHOLDER_TYPE_NOT_SUPPORTED_NOT_RECOVERABLE: Placeholder._Type.ValueType  # 4

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    PLACEHOLDER_TYPE_NONE: Placeholder.Type.ValueType  # 0
    PLACEHOLDER_TYPE_DECRYPTION_FAILURE: Placeholder.Type.ValueType  # 1
    PLACEHOLDER_TYPE_NOT_SUPPORTED_NEED_UPDATE: Placeholder.Type.ValueType  # 2
    PLACEHOLDER_TYPE_DEVICE_UNAVAILABLE: Placeholder.Type.ValueType  # 3
    PLACEHOLDER_TYPE_NOT_SUPPORTED_NOT_RECOVERABLE: Placeholder.Type.ValueType  # 4

    PLACEHOLDERTYPE_FIELD_NUMBER: builtins.int
    placeholderType: global___Placeholder.Type.ValueType
    def __init__(
        self,
        *,
        placeholderType: global___Placeholder.Type.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["placeholderType", b"placeholderType"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["placeholderType", b"placeholderType"]) -> None: ...

global___Placeholder = Placeholder



import asyncio
import json
import os
import tempfile
import threading
import uuid
from io import BytesIO

import magic
from PIL import Image, ImageSequence

from ..exc import ConvertStickerError
from .calc import auto_sticker, original_sticker
from .ffmpeg import AFFmpeg, FFmpeg
from .iofile import (
    TemporaryFile,
    get_bytes_from_name_or_url,
    get_bytes_from_name_or_url_async,
)
from .platform import is_executable_installed


def add_exif(name: str = "", packname: str = "") -> bytes:
    """
    Adds EXIF metadata to a sticker pack.

    :param name: Name of the sticker pack, defaults to an empty string.
    :type name: str, optional
    :param packname: Publisher of the sticker pack, defaults to an empty string.
    :type packname: str, optional
    :return: Byte array containing the EXIF metadata.
    :rtype: bytes
    """
    json_data = {
        "sticker-pack-id": "com.snowcorp.stickerly.android.stickercontentprovider b5e7275f-f1de-4137-961f-57becfad34f2",
        "sticker-pack-name": name,
        "sticker-pack-publisher": packname,
        "android-app-store-link": "https://play.google.com/store/apps/details?id=com.marsvard.stickermakerforwhatsapp",
        "ios-app-store-link": "https://itunes.apple.com/app/sticker-maker-studio/id1443326857",
    }

    exif_attr = bytes.fromhex(
        "49 49 2A 00 08 00 00 00 01 00 41 57 07 00 00 00 00 00 16 00 00 00"
    )
    json_buffer = json.dumps(json_data).encode("utf-8")
    exif = exif_attr + json_buffer
    exif_length = len(json_buffer)
    exif = exif[:14] + exif_length.to_bytes(4, "little") + exif[18:]
    return exif


def webpmux_is_installed():
    return is_executable_installed("webpmux")


MAX_STICKER_SIZE = 512000
WEBPMUX_IS_AVAILABLE = False
if webpmux_is_installed():
    MAX_STICKER_SIZE = 712000
    WEBPMUX_IS_AVAILABLE = True


async def aio_convert_to_sticker(
    file: bytes,
    name="",
    packname="",
    enforce_not_broken=False,
    animated_gif=False,
    is_webm=False,
):
    async with AFFmpeg(file) as ffmpeg:
        sticker = await ffmpeg.cv_to_webp(
            enforce_not_broken=enforce_not_broken,
            animated_gif=animated_gif,
            max_sticker_size=MAX_STICKER_SIZE,
            is_webm=is_webm,
        )
    if not WEBPMUX_IS_AVAILABLE:
        return sticker, False

    exif_filename = TemporaryFile(prefix=None, touch=False).__enter__()
    with open(exif_filename.path, "wb") as file:
        file.write(add_exif(name=name, packname=packname))
    temp = tempfile.gettempdir() + "/" + f"{uuid.uuid4()}" + ".webp"
    async with AFFmpeg(sticker) as ffmpeg:
        cmd = [
            "webpmux",
            "-set",
            "exif",
            f"{exif_filename.path}",
            ffmpeg.filepath,
            "-o",
            temp,
        ]
        await ffmpeg.call(cmd)
    exif_filename.__exit__(None, None, None)
    with open(temp, "rb") as file:
        buf = file.read()
    os.remove(temp)
    return buf, True


def convert_to_sticker(
    file: bytes,
    name="",
    packname="",
    enforce_not_broken=False,
    animated_gif=False,
    is_webm=False,
):
    with FFmpeg(file) as ffmpeg:
        sticker = ffmpeg.cv_to_webp(
            enforce_not_broken=enforce_not_broken,
            animated_gif=animated_gif,
            max_sticker_size=MAX_STICKER_SIZE,
            is_webm=is_webm,
        )
    if not WEBPMUX_IS_AVAILABLE:
        return sticker, False

    exif_filename = TemporaryFile(prefix=None, touch=False).__enter__()
    with open(exif_filename.path, "wb") as file:
        file.write(add_exif(name=name, packname=packname))
    temp = tempfile.gettempdir() + "/" + f"{uuid.uuid4()}" + ".webp"
    with FFmpeg(sticker) as ffmpeg:
        cmd = [
            "webpmux",
            "-set",
            "exif",
            f"{exif_filename.path}",
            ffmpeg.filepath,
            "-o",
            temp,
        ]
        ffmpeg.call(cmd)
    exif_filename.__exit__(None, None, None)
    with open(temp, "rb") as file:
        buf = file.read()
    os.remove(temp)
    return buf, True


astick_sem = asyncio.Semaphore(20)


async def aio_convert_to_webp(
    sticker, name, packname, crop=False, passthrough=True, transparent=False
):
    sticker = await get_bytes_from_name_or_url_async(sticker)
    animated = is_webm = is_image = saved_exif = stk = False
    mime = magic.from_buffer(sticker, mime=True)
    if mime == "image/webp":
        io_save = BytesIO(sticker)
        img = Image.open(io_save)
        if len(ImageSequence.all_frames(img)) < 2:
            is_image = True
    elif passthrough:
        raise ConvertStickerError(
            "File is not a webp, which is required for passthrough."
        )
    elif mime == "video/webm":
        is_webm = True
    elif (mime := mime.split("/"))[0] == "image":
        is_image = True
    animated = not is_image
    if passthrough:
        return sticker, animated
    if is_image:
        io_save = BytesIO(sticker)
        stk = auto_sticker(io_save) if crop else original_sticker(io_save)
        io_save = BytesIO()
        # io_save.seek(0)
    else:
        animated = True
        async with astick_sem:
            sticker, saved_exif = await aio_convert_to_sticker(
                sticker,
                name,
                packname,
                enforce_not_broken=True,
                animated_gif=transparent,
                is_webm=is_webm,
            )
        if saved_exif:
            io_save = BytesIO(sticker)
        else:
            stk = Image.open(BytesIO(sticker))
            io_save = BytesIO()
    if not saved_exif:
        stk.save(
            io_save,
            format="webp",
            exif=add_exif(name, packname),
            save_all=True,
            loop=0,
        )
    return io_save.getvalue(), animated


stick_sem = threading.Semaphore(20)


def convert_to_webp(
    sticker, name, packname, crop=False, passthrough=True, transparent=False
):
    sticker = get_bytes_from_name_or_url(sticker)
    animated = is_webm = is_image = saved_exif = stk = False
    mime = magic.from_buffer(sticker, mime=True)
    if mime == "image/webp":
        io_save = BytesIO(sticker)
        img = Image.open(io_save)
        if len(ImageSequence.all_frames(img)) < 2:
            is_image = True
    elif passthrough:
        raise ConvertStickerError(
            "File is not a webp, which is required for passthrough."
        )
    elif mime == "video/webm":
        is_webm = True
    elif (mime := mime.split("/"))[0] == "image":
        is_image = True
    animated = not is_image
    if passthrough:
        return sticker, animated
    if is_image:
        io_save = BytesIO(sticker)
        stk = auto_sticker(io_save) if crop else original_sticker(io_save)
        io_save = BytesIO()
        # io_save.seek(0)
    else:
        animated = True
        with stick_sem:
            sticker, saved_exif = convert_to_sticker(
                sticker,
                name,
                packname,
                enforce_not_broken=True,
                animated_gif=transparent,
                is_webm=is_webm,
            )
        if saved_exif:
            io_save = BytesIO(sticker)
        else:
            stk = Image.open(BytesIO(sticker))
            io_save = BytesIO()
    if not saved_exif:
        stk.save(
            io_save,
            format="webp",
            exif=add_exif(name, packname),
            save_all=True,
            loop=0,
        )
    return io_save.getvalue(), animated
